# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '../..'))


import cocos
from cocos.director import director

import pyglet
from pyglet import font, image
from pyglet.gl import *
from pyglet.window import key

from cocos.actions import *
from cocos.layer import *
from cocos.scenes.transitions import *
from cocos.sprite import *
from cocos import text


class HelloWorld(Layer):
    def __init__(self):

        super( HelloWorld, self ).__init__()

        # see pyglet documentation for help on this lines
        self.text = pyglet.text.Label('Hello, World!', font_name='', font_size=32, x=100, y=240, batch=self.batch)

        
class BackgroundLayer(Layer):
    """
    """
    def __init__( self, path_name ):
        super(BackgroundLayer, self).__init__()
        
        self.image = image.load(path_name)
        
        
    def draw(self):
        texture = self.image.texture
        
        rx = director.window.width - 2*director._offset_x
        ry = director.window.height - 2*director._offset_y
        
        tx = float(rx)/texture.width
        ty = float(ry)/texture.height


        glEnable(GL_TEXTURE_2D)        
        glBindTexture(texture.target, texture.id)

        x, y = director.get_window_size()
        glBegin(gl.GL_QUADS)
        glTexCoord2d(0,0);
        glVertex2f( 0, 0 )
        glTexCoord2d(0,ty);
        glVertex2f( 0, y )
        glTexCoord2d(tx,ty);
        glVertex2f( x, y )
        glTexCoord2d(tx,0);
        glVertex2f( x,0 )
        glEnd()
        
        glDisable(GL_TEXTURE_2D)
        

class SpriteLayer( Layer ):
    pass

class SpriteMoveTo( SpriteLayer ):
    def on_enter( self ):
        super(SpriteMoveTo, self).on_enter()
        
        self.stop()

        sprite = Sprite("grossini.png", (20,100))
        self.add( sprite )

        sprite.do( MoveTo( (580,100), 3 ) )

    def on_exit(self):
        super(SpriteMoveTo, self).on_exit()
        [ self.remove( c ) for c in self.get_chidren() ]
        


class TitleSubTitleLayer(cocos.layer.Layer):
    def __init__(self, title, subtitle):
        super(TitleSubTitleLayer, self).__init__()

        x, y = director.get_window_size()

        self.title = text.Label(
                title, (x/2, y/2+50), font_name = 'Gill Sans',
                font_size = 64, anchor_x='center', anchor_y='center' )
        self.add( self.title )
        
        self.subtitle = text.Label(
                subtitle, (x/2, y/2-30), font_name = 'Gill Sans',
                font_size = 44, anchor_x='center', anchor_y='center' )
        self.add( self.subtitle )
        
class BulletListLayer(cocos.layer.Layer):
    def __init__(self, title, lines):
        super(BulletListLayer, self).__init__()
        x, y = director.get_window_size()


        self.title = text.Label(
                title, (x/2, y-50), font_name = 'Gill Sans',
                font_size = 64, anchor_x='center', anchor_y='center' )
        self.add( self.title )

        start_y = (y/12)*8
        font_size = 52 / (len(lines)/2.2+1)
        font_size = min(font_size, 52)
        line_font = font.load('Gill Sans', font_size)
        tot_height = 0
        max_width = 0
        rendered_lines = []
        step = 300/ max(len(lines),1)
        i = 0
        for line in lines:
            line_text = text.Label(
                line, (x/2, y-150-step*i), font_name = 'Gill Sans',
                font_size = font_size, anchor_x='center', anchor_y='center' )
            i += 1   
            self.add( line_text )    
class TransitionControl(cocos.layer.Layer):
    is_event_handler = True
    
    def __init__(self, scenes, transitions=None):
        super(TransitionControl, self).__init__()
        
        self.transitions = transitions
        self.scenes = scenes
        for scene in scenes:
            if not self in scene.get_children():
                scene.add(self)
                
        self.scene_p = 0
        
            
    def next_scene(self):
        self.scene_p +=1 
        if self.scene_p >= len(self.scenes):
            self.scene_p = len(self.scenes)-1
        else:
            self.transition(self.transitions[self.scene_p%len(self.transitions)])
    
    def prev_scene(self):
        self.scene_p -=1 
        if self.scene_p < 0:
            self.scene_p = 0
        else:
            self.transition()
           
    def transition(self, transition=None):
        if transition:
            director.replace( transition(
                        self.scenes[ self.scene_p ],
                        duration = 1
                         )
                )
        else:
            director.replace( self.scenes[ self.scene_p ] )
        
    def on_key_press(self, keyp, mod):
        if keyp in (key.PAGEDOWN,):
            self.next_scene()
        elif keyp in (key.PAGEUP,):
            self.prev_scene()
            
class RunScene(cocos.layer.Layer):
    is_event_handler = True
    
    def __init__(self, target):
        super(RunScene, self).__init__()
        
        self.target = target
        
    def on_key_press(self, keyp, mod):
        if keyp in (key.F1,):
            director.push( self.target )    
        
class ControlLayer(cocos.layer.Layer):
    is_event_handler = True
    
    def on_enter( self ):
        super(ControlLayer, self).on_enter()
        
        ft_title = font.load( "Arial", 32 )
        ft_subtitle = font.load( "Arial", 18 )
        ft_help = font.load( "Arial", 16 )

        self.text_title = font.Text(ft_title, "Transition Demos",
            x=5,
            y=480,
            anchor_x=font.Text.LEFT,
            anchor_y=font.Text.TOP)

        self.text_subtitle = font.Text(ft_subtitle, transition_list[current_transition].__name__,
            x=5,
            y=400,
            anchor_x=font.Text.LEFT,
            anchor_y=font.Text.TOP)
        
        self.text_help = font.Text(ft_help,"Press LEFT / RIGHT for prev/next example, ENTER to restart example",
            x=320,
            y=20,
            anchor_x=font.Text.CENTER,
            anchor_y=font.Text.CENTER)

    def step( self, df ):
        self.text_help.draw()

        self.text_subtitle.text = transition_list[current_transition].__name__
        self.text_subtitle.draw()
        self.text_title.draw()

    def on_key_press( self, k , m ):
        global current_transition, control_p
        if k == key.LEFT:
            current_transition = (current_transition-1)%len(transition_list)
        if k == key.RIGHT:
            current_transition = (current_transition+1)%len(transition_list)
        if k == key.ENTER:
            director.replace( transition_list[current_transition](
                        control_list[control_p],
                        (control_list[(control_p+1)%len(control_list)] ),
                        2)                                
                    )
            control_p = (control_p + 1) % len(control_list)
            return True
        if k == key.ESCAPE:
            director.scene.end()
            return True

class GrossiniLayer(cocos.layer.Layer):
    def __init__( self ):
        super( GrossiniLayer, self ).__init__()

        g = Sprite('grossini.png', (320,240))

        self.add( g )

        rot = Rotate( 180, 5 )

        g.do( Repeat( rot ) )

   
class GrossiniLayer2(cocos.layer.Layer):
    def __init__( self ):
        super( GrossiniLayer2, self ).__init__()

        rot = Rotate( 180, 5 )

        g = Sprite('grossinis_sister1.png', (490,240))
        self.add( g )
        g.do( Repeat( rot ) )

        g = Sprite('grossinis_sister2.png', (150,240))
        self.add( g )
        g.do( Repeat( rot ) )

if __name__ == "__main__":
    aspect = 1280 / float(800)
    director.init( resizable=True, width=640, height=480 ) 
    director.window.set_fullscreen(False)
    x,y = director.get_window_size()
    #background = BackgroundLayer("background.png")
    #background = BackgroundLayer("coconut.jpg")
    background = cocos.layer.ColorLayer(0,0,0,255)
    
    transition_list = [
        JumpZoomTransition
        ]
    current_transition = 0

    scenes = [
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            TitleSubTitleLayer("cocos2d", "a 2d game library"),
        ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("cocos2d", [
                "una libreria para hacer juegos 2d",
                "programando en python",
                ])
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Juegos y python", [
                "Juegos en un lenguaje interpretado?",
                "Depende del juego",
                "Hay mucho que se puede hacer en 30ms",
                ])
            ),
        
        
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Historia", [
                "2000 : pygame",
                "2005 : pyweek",
                "2006 : pyglet",
                "2008 : cocos2d"
                ])
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Features", [
                "Control de flujo",
                "Sprites",
                "Acciones",
                "Efectos",
                "Transiciones",
                "Menus", 
                ])
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Features (2)", [
                "Texto / HTML",
                "Tiles"
                "Bien Documentado",
                "Interprete de python incluido",
                "Licencia BSD",
                "Basado en pyglet",
                "OpenGL",
                ]),
            ),
        cocos.scene.Scene ( cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Conceptos", []).add(
                Sprite("scene_sp.png", (x/2, 100))),
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Documentacion", [
                "Tutoriales en video",
                "Guia de programacion (1KLOT)",
                "Documentacion de api [casi] completa",
                "5KLOC de tests",
                "Los test son ejemplos funcionales",
                "FAQ [en progreso]",

            ]),
            ),
            
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Comunidad", [
                "5 juegos terminados",
                "Varios proyectos en desarrollo",
                "Contribuidores de todo el mundo",
                "Lista de correo 'funcional'",
            ]),
            ),
        cocos.scene.Scene ( cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Control De Flujo", []),
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Sprites", []),
            ),
        cocos.scene.Scene ( cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Acciones", []),
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Efectos", []),
            ),
        cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            BulletListLayer("Transiciones", []),
            ),
        
         cocos.scene.Scene (cocos.layer.ColorLayer(0,0,0,255),
            TitleSubTitleLayer("cocos2d", "http://www.cocos2d.org"),
        ),
        ]
    transitions = [None]*(len(scenes)-1)
    all_t = ['RotoZoomTransition','JumpZoomTransition',

            'SlideInLTransition','SlideInRTransition',
            'SlideInBTransition','SlideInTTransition',

            'FlipX3DTransition', 'FlipY3DTransition','FlipAngular3DTransition',

            'ShuffleTransition',
            'TurnOffTilesTransition',
            'FadeTRTransition', 'FadeBLTransition',
            'FadeUpTransition', 'FadeDownTransition',

            'ShrinkGrowTransition',
            'CornerMoveTransition',
            'EnvelopeTransition',

            'SplitRowsTransition', 'SplitColsTransition',

            'FadeTransition',]
    transitions = [ getattr(cocos.scenes.transitions, all_t[i % len(all_t)]) 
                for i in range(len(scenes)-1) ]
                
    TransitionControl( scenes, transitions )

    def color_name_scene(name, color):    
        return cocos.scene.Scene(
            cocos.layer.ColorLayer(*color).add(
                cocos.text.Label(
                    name, (x/2,y/2), 
                    font_name = 'Gill Sans', font_size = 64, 
                    anchor_x='center', anchor_y='center'
                )
            )
        )
    director.interpreter_locals["uno"] = color_name_scene("uno", (255,0,0,255))
    director.interpreter_locals["dos"] = color_name_scene("dos", (0,255,0,255))
    director.interpreter_locals["tres"] = color_name_scene("tres", (0,0,255,255))
    
    
    director.run (scenes[0])




# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#

import cocos
from cocos.director import director

import pyglet

class KeyDisplay(cocos.layer.Layer):

    is_event_handler = True     #: enable pyglet's events

    def __init__(self):

        super( KeyDisplay, self ).__init__()

        self.text = cocos.text.Label("", x=100, y=280 )

        # To keep track of which keys are pressed:
        self.keys_pressed = set()
        self.update_text()
        self.add(self.text) 

    def update_text(self):
        key_names = [pyglet.window.key.symbol_string (k) for k in self.keys_pressed]
        text = 'Keys: '+','.join (key_names)
        # Update self.text
        self.text.element.text = text

    def on_key_press (self, key, modifiers):
        """This function is called when a key is pressed.
        
        'key' is a constant indicating which key was pressed.
        'modifiers' is a bitwise or of several constants indicating which
           modifiers are active at the time of the press (ctrl, shift, capslock, etc.)
            
        """
        self.keys_pressed.add (key)
        self.update_text()

    def on_key_release (self, key, modifiers):
        """This function is called when a key is released.
        
        'key' is a constant indicating which key was pressed.
        'modifiers' is a bitwise or of several constants indicating which
           modifiers are active at the time of the press (ctrl, shift, capslock, etc.)

        Constants are the ones from pyglet.window.key
        """
        self.keys_pressed.remove (key)
        self.update_text()

class MouseDisplay(cocos.layer.Layer):

    # If you want that your layer receives events
    # you must set this variable to 'True',
    # otherwise it won't receive any event.
    is_event_handler = True

    def __init__(self):
        super( MouseDisplay, self ).__init__()

        self.posx = 100
        self.posy = 240
        self.text = cocos.text.Label('No mouse events yet', font_size=18, x=self.posx, y=self.posy )
        self.add( self.text )

    def update_text (self, x, y):
        text = 'Mouse @ %d,%d' % (x, y)
        self.text.element.text = text
        self.text.element.x = self.posx
        self.text.element.y = self.posy


    def on_mouse_motion (self, x, y, dx, dy):
        """This function is called when the mouse is moved over the app.
        
        (x, y) are the physical coordinates of the mouse
        (dx, dy) is the distance vector covered by the mouse pointer since the
          last call.
        """
        self.update_text (x, y)

    def on_mouse_press (self, x, y, buttons, modifiers):
        """This function is called when any mouse button is pressed

        (x, y) are the physical coordinates of the mouse
        'buttons' is a bitwise or of pyglet.window.mouse constants LEFT, MIDDLE, RIGHT
        'modifiers' is a bitwise or of pyglet.window.key modifier constants
           (values like 'SHIFT', 'OPTION', 'ALT')
        """
        self.posx, self.posy = director.get_virtual_coordinates (x, y)
        self.update_text (x,y)


if __name__ == "__main__":
    director.init(resizable=True)
    # Run a scene with our event displayers:
    director.run( cocos.scene.Scene( KeyDisplay(), MouseDisplay() ) )



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos import draw
import pyglet

import random
ri = random.randint

class TestFigure(draw.Canvas):
    def render(self):
        x,y = director.get_window_size()
        ys = y/4
        ye = ys*3
        xs = x/4
        line_width = 50
        self.set_color( (255,255,0,50) )
        self.set_stroke_width( line_width )
        
        self.set_join( draw.BEVEL_JOIN )
        self.move_to( (xs+100, ys) )
        self.line_to( (xs*2+100,ys) )
        self.line_to( (xs*2+100,ys*2) )
        self.line_to( (xs*3+100,ys*2) )
        
        self.set_join( draw.MITER_JOIN )
        self.move_to( (xs, ys+100) )
        self.line_to( (xs*2,ys+100) )
        self.line_to( (xs*2,ys*2+100) )
        self.line_to( (xs*3,ys*2+100) )
        
        self.set_join( draw.ROUND_JOIN )
        self.move_to( (xs-100, ys+200) )
        self.line_to( (xs*2-100,ys+200) )
        self.line_to( (xs*2-100,ys*2+200) )
        self.line_to( (xs*3-100,ys*2+200) )
        
        
            
        
class TestLayer(cocos.layer.Layer):
    def __init__(self):
        super( TestLayer, self ).__init__()
        
        self.add( TestFigure() )        
        self.schedule( lambda x: 0 )

if __name__ == "__main__":
    director.init()
    test_layer = TestLayer ()
    main_scene = cocos.scene.Scene (test_layer)
    director.run (main_scene)




# ----------------------------------------------------------------------------
# cocos2d
# Copyright (c) 2008 Daniel Moisset, Ricardo Quesada, Rayentray Tappa, Lucio Torre
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of cocos2d nor the names of its
#     contributors may be used to endorse or promote products
#     derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Action Sprite

Animating a sprite
==================

To execute any action you need to create an action::

    move = MoveBy( (50,0), 5 )

In this case, ``move`` is an action that will move the sprite
50 pixels to the right (``x`` coordinate) and  0 pixel in the ``y`` coordinate
in 5 seconds.

And now tell the sprite to execute it::

    sprite.do( move )
'''

__docformat__ = 'restructuredtext'

import cocosnode
from batch import *

import pyglet
from pyglet import image
from pyglet.gl import *

__all__ = [ 'Sprite',                     # Sprite class
            ]



class Sprite( BatchableNode, pyglet.sprite.Sprite):
    '''Sprites are sprites that can execute actions.

    Example::

        sprite = Sprite('grossini.png')
    '''

    def __init__( self, image, position=(0,0), rotation=0, scale=1, opacity = 255, color=(255,255,255), anchor = None ):
        '''Initialize the sprite

        :Parameters:
                `image` : string or image
                    name of the image resource or a pyglet image.
                `position` : tuple
                    position of the anchor. Defaults to (0,0)
                `rotation` : float
                    the rotation (degrees). Defaults to 0.
                `scale` : float
                    the zoom factor. Defaults to 1.
                `opacity` : int
                    the opacity (0=transparent, 255=opaque). Defaults to 255.
                `color` : tuple
                    the color to colorize the child (RGB 3-tuple). Defaults to (255,255,255).
                `anchor` : (float, float)
                    (x,y)-point from where the image will be positions, rotated and scaled in pixels. For example (image.width/2, image.height/2) is the center (default).
        '''

        if isinstance(image, str):
            image = pyglet.resource.image(image)


        pyglet.sprite.Sprite.__init__(self, image)
        cocosnode.CocosNode.__init__(self)

        if anchor is None:
            if isinstance(self.image, pyglet.image.Animation):
                anchor = (image.frames[0].image.width / 2,
                    image.frames[0].image.height / 2)
            else:
                anchor = image.width / 2, image.height / 2


        self.image_anchor = anchor
        self.anchor = (0, 0)

        #: group.
        #: XXX what is this?
        self.group = None

        #: children group.
        #: XXX what is this ?
        self.children_group = None

        #: position of the sprite in (x,y) coordinates
        self.position = position

        #: rotation degrees of the sprite. Default: 0 degrees
        self.rotation = rotation

        #: scale of the sprite where 1.0 the default value
        self.scale = scale

        #: opacity of the sprite where 0 is transparent and 255 is solid
        self.opacity = opacity

        #: color of the sprite in R,G,B format where 0,0,0 is black and 255,255,255 is white
        self.color = color


    def _set_anchor_x(self, value):
        if isinstance(self.image, pyglet.image.Animation):
            for img in self.image.frames:
                img.image.anchor_x = value
            self._texture.anchor_x = value
        else:
            self.image.anchor_x = value
        self._update_position()

    def _get_anchor_x(self):
        if isinstance(self.image, pyglet.image.Animation):
            return self.image.frames[0].image.anchor_x
        else:
            return self.image.anchor_x
    image_anchor_x = property(_get_anchor_x, _set_anchor_x)

    def _set_anchor_y(self, value):
        if isinstance(self.image, pyglet.image.Animation):
            for img in self.image.frames:
                img.image.anchor_y = value
            self._texture.anchor_y = value
        else:
            self.image.anchor_y = value
        self._update_position()

    def _get_anchor_y(self):
        if isinstance(self.image, pyglet.image.Animation):
            return self.image.frames[0].image.anchor_y
        else:
            return self.image.anchor_y
    image_anchor_y = property(_get_anchor_y, _set_anchor_y)

    def _set_anchor(self, value):
        self._set_anchor_x(value[0])
        self._set_anchor_y(value[1])

    def _get_anchor(self):
        return (self._get_anchor_x(), self._get_anchor_y())

    image_anchor = property(_get_anchor, _set_anchor)

    def draw(self):
        self._group.set_state()
        if self._vertex_list is not None:
            self._vertex_list.draw(GL_QUADS)
        self._group.unset_state()

Sprite.supported_classes = Sprite



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos.actions import *
import pyglet
        

if __name__ == "__main__":
    director.init()
    bg_layer = cocos.layer.ColorLayer(255,0,0,255)
    translate_layer = cocos.layer.Layer()
    x, y = director.get_window_size()
    sub = cocos.scene.Scene( bg_layer )
    sub.do( MoveBy( (x/2, y/2), 5) )
    sub.do( ScaleBy( 1/2.1, 5) )
    main_scene = cocos.scene.Scene (sub)
    director.run (main_scene)



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#

import pyglet
import cocos
from cocos.director import director
from cocos.actions import *
from cocos.layer import *

from pyglet.gl import *



class BackgroundLayer( cocos.layer.Layer ):
    def __init__(self):
        super( BackgroundLayer, self ).__init__()
        self.img = pyglet.resource.image('background_image.png')

    def draw( self ):
        glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        glPopMatrix()

if __name__ == "__main__":
    director.init( resizable=True )
#    director.set_depth_test()

    main_scene = cocos.scene.Scene()

    background = BackgroundLayer()
    color = ColorLayer(255,32,32,128)

    main_scene.add( background, z=0 )
    main_scene.add( color, z=1 )

    # use the remaining grid and move it's camera
    rot = OrbitCamera( angle_x=45,  angle_z=0, delta_z=360, duration=8 )
    rot2 = OrbitCamera( radius=2, delta_radius=-1, angle_x=0, angle_z=0, delta_z=360, duration=8 )

    background.do( rot + Reverse(rot) )
    color.do( rot2 + Reverse(rot2) )

    director.run (main_scene)



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos.actions import *

import pyglet

class TestLayer(cocos.layer.Layer):
    def __init__(self):
        super( TestLayer, self ).__init__()
        
        x,y = director.get_window_size()
        
        self.sprite = Sprite('grossini.png', (x/2, y/2))
        self.sprite2 = Sprite('grossini.png', (x/4, y/2))
        self.add( self.sprite  )
        self.add( self.sprite2, name="grossini"  )
        
        def rem():
            self.remove( self.sprite )
            self.remove( "grossini" )
        self.do( Delay(5) + CallFunc( rem ) )
            
    
        

if __name__ == "__main__":
    director.init()
    test_layer = TestLayer ()
    main_scene = cocos.scene.Scene (test_layer)
    director.run (main_scene)



# ----------------------------------------------------------------------------
# cocos2d
# Copyright (c) 2008 Daniel Moisset, Ricardo Quesada, Rayentray Tappa, Lucio Torre
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of cocos2d nor the names of its
#     contributors may be used to endorse or promote products
#     derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Implementation of QuadMoveBy actions

These actions modifies the x and y coordinates of fixed-size grid of (1,1).
The z-coordinate is not modified.
'''
__docformat__ = 'restructuredtext'

import math

from cocos.director import director
from cocos.euclid import *
from basegrid_actions import *

__all__ = ['QuadMoveBy',
           'MoveCornerUp',
           'MoveCornerDown',
           'CornerSwap',
           'Flip',
           'FlipX',
           'FlipY',
           'SkewHorizontal',
           'SkewVertical',
           ]


class QuadMoveBy( Grid3DAction ):
    '''QuadMoveBy moves each vertex of the grid. The size of the grid is (1,1)

    Vertex positions::

        vertex3 --<-- vertex2
            |            |
            v            ^
            |            |
        vertex0 -->-- vertex1
        
        The vertices will move from the origin (src parameters) a relative distance (delta parameters) in duration time.

    Example::

       scene.do( QuadMoveBy( src0, src1, src2, src3,
               delta0, delta1, delta2, delta3,
               duration) )


       '''

    def init( self, 
              src0=(0,0), src1=(-1,-1), src2=(-1,-1), src3=(-1,-1),
              delta0=(0,0), delta1=(0,0), delta2=(0,0), delta3=(0,0),
              grid=(1,1),
              *args, **kw ):
        '''Initializes the QuadMoveBy
        
        :Parameters:
            `src0` : (int, int)
                Initial value for the bottom-left coordinate. Default is (0,0)
            `src1` : (int, int)
                Initial value for the bottom-right coordinate. Default is (win_size_x,0)
            `src2` : (int, int)
                Initial value for the upper-right coordinate. Default is (win_size_x, win_size_y)
            `src3` : (int, int)
                Initial value for the upper-left coordinate. Default is (0, win_size_y)
            `delta0` : (int, int)
                The bottom-left relative coordinate. Default is (0,0)
            `delta1` : (int, int)
                The bottom-right relative coordinate. Default is (0,0)
            `delta2` : (int, int)
                The upper-right relative coordinate. Default is (0,0)
            `delta3` : (int, int)
                The upper-left relative coordinate. Default is (0,0)
        '''

        if grid != (1,1):
            raise GridException("Invalid grid size.")

        super( QuadMoveBy, self).init( grid, *args, **kw )
        
        x,y = director.get_window_size()
        
        if src1 == (-1,-1):
            src1 = ( x,0 )
        if src2 == (-1,-1):
            src2 = (x,y)
        if src3 == (-1,-1):
            src3 = (0,y)  

        self.src0 = Point3( src0[0], src0[1], 0 )
        self.src1 = Point3( src1[0], src1[1], 0 )  
        self.src2 = Point3( src2[0], src2[1], 0 )
        self.src3 = Point3( src3[0], src3[1], 0)
        self.delta0 = Point3( delta0[0], delta0[1], 0)
        self.delta1 = Point3( delta1[0], delta1[1], 0) 
        self.delta2 = Point3( delta2[0], delta2[1], 0)
        self.delta3 = Point3( delta3[0], delta3[1], 0)
       
    def update( self, t ):
        new_pos0 = self.src0 + self.delta0 * t
        new_pos1 = self.src1 + self.delta1 * t
        new_pos2 = self.src2 + self.delta2 * t
        new_pos3 = self.src3 + self.delta3 * t

        self.set_vertex( 0,0, new_pos0 )
        self.set_vertex( 1,0, new_pos1 )
        self.set_vertex( 1,1, new_pos2 )
        self.set_vertex( 0,1, new_pos3 )


class MoveCornerUp( QuadMoveBy ):
    '''MoveCornerUp moves the bottom-right corner to the upper-left corner in duration time'''
    def __init__(self, *args, **kw):
        x,y = director.get_window_size()
        super(MoveCornerUp, self).__init__( delta1=(-x,y), *args, **kw )

class MoveCornerDown( QuadMoveBy ):
    '''MoveCornerDown moves the upper-left corner to the bottom-right corner in duration time'''
    def __init__(self, *args, **kw):
        x,y = director.get_window_size()
        super(MoveCornerDown, self).__init__( delta3=(x,-y), *args, **kw )

class CornerSwap( QuadMoveBy ):
    '''CornerSwap moves the upper-left corner to the bottom-right corner in vice-versa in duration time.
    The resulting effect is a reflection + rotation.
    '''
    def __init__(self, *args, **kw):
        x,y = director.get_window_size()
        super(CornerSwap, self).__init__( delta1=(-x,y), delta3=(x,-y), *args, **kw )

class Flip( QuadMoveBy ):
    '''Flip moves the upper-left corner to the bottom-left corner and vice-versa, and
    moves the upper-right corner to the bottom-left corner and vice-versa, flipping the
    window upside-down, and right-left.
    '''
    def __init__(self, *args, **kw):
        x,y = director.get_window_size()
        super(Flip, self).__init__( delta0=(x,y), delta1=(-x,y), delta2=(-x,-y), delta3=(x,-y), *args, **kw )

class FlipX( QuadMoveBy ):
    '''FlipX flips the screen horizontally, moving the left corners to the right, and vice-versa.
    '''
    def __init__(self, *args, **kw):
        x,y = director.get_window_size()
        super(FlipX, self).__init__( delta0=(x,0), delta1=(-x,0), delta2=(-x,0), delta3=(x,0), *args, **kw )

class FlipY( QuadMoveBy ):
    '''FlipY flips the screen vertically, moving the upper corners to the bottom, and vice-versa.
    '''
    def __init__(self, *args, **kw):
        x,y = director.get_window_size()
        super(FlipY, self).__init__( delta0=(0,y), delta1=(0,y), delta2=(0,-y), delta3=(0,-y), *args, **kw )

class SkewHorizontal( QuadMoveBy ):
    '''SkewHorizontal skews the screen horizontally. default skew: x/3'''
    def __init__(self, delta=None, *args, **kw):
        x,y = director.get_window_size()
        if delta==None:
            delta=x//3
        super(SkewHorizontal, self).__init__( delta1=(-delta,0), delta3=(delta,0), *args, **kw )

class SkewVertical( QuadMoveBy ):
    '''SkewVertical skews the screen vertically. default skew: y/3'''
    def __init__(self, delta=None, *args, **kw):
        x,y = director.get_window_size()
        if delta==None:
            delta=y//3
        super(SkewVertical, self).__init__( delta0=(0,delta), delta2=(0,-delta), *args, **kw )




import pyglet

class Colors( object ):
    colors = ['black','orange','red','yellow','cyan','magenta','green','blue',
            'black',        # don't remove
            'rotate','scale','liquid','waves','twirl','lens',
            'black' ]       # don't remove

    BLACK,ORANGE,RED,YELLOW,CYAN,MAGENTA,GREEN,BLUE, LAST_COLOR,  \
        ROTATE, SCALE, LIQUID, WAVES, TWIRL, LENS, LAST_SPECIAL = range( len(colors) )

    images = [ pyglet.resource.image('block_%s.png' % color) for color in colors ]
    
    specials = [ k for k in range( LAST_COLOR+1, LAST_SPECIAL) ]




from colors import *

class Level( object ):
    pass

class LevelLens( Level ):
    speed = 0.5
    blocks = [ Colors.LENS ]
    lines = 10
    prob = 0.07

class LevelScale( Level ):
    speed = 0.5
    blocks = [ Colors.SCALE ]
    lines = 10
    prob = 0.07

class LevelLiquid( Level ):
    speed = 0.45
    blocks = [ Colors.LIQUID ]
    lines = 12
    prob = 0.07

class LevelWaves( Level ):
    speed = 0.4
    blocks = [ Colors.WAVES ]
    lines = 14
    prob = 0.09

class LevelTwirl( Level ):
    speed = 0.35
    blocks = [ Colors.TWIRL ]
    lines = 16
    prob = 0.11

class LevelRotate( Level ):
    speed = 0.3
    blocks = [ Colors.ROTATE ]
    lines = 18
    prob = 0.13

class LevelTwirlZoom( Level ):
    speed = 0.25
    blocks = [ Colors.TWIRL, Colors.SCALE ]
    lines = 20
    prob = 0.15

class LevelWavesRot( Level ):
    speed = 0.2
    blocks = [ Colors.WAVES, Colors.ROTATE]
    lines = 22
    prob = 0.17

levels = [ LevelLens, LevelScale, LevelLiquid, LevelWaves, LevelTwirl, LevelRotate, LevelTwirlZoom, LevelWavesRot ]



# ----------------------------------------------------------------------------
# cocos2d
# Copyright (c) 2008 Daniel Moisset, Ricardo Quesada, Rayentray Tappa, Lucio Torre
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of cocos2d nor the names of its
#     contributors may be used to endorse or promote products
#     derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Transitions between Scenes'''

__docformat__ = 'restructuredtext'

import pyglet

from cocos.actions import *
import cocos.scene as scene
from cocos.director import director
from cocos.layer import ColorLayer
from cocos.sprite import Sprite

__all__ = [ 'TransitionScene',
            'RotoZoomTransition','JumpZoomTransition',

            'MoveInLTransition','MoveInRTransition',
            'MoveInBTransition','MoveInTTransition',
            'SlideInLTransition','SlideInRTransition',
            'SlideInBTransition','SlideInTTransition',

            'FlipX3DTransition', 'FlipY3DTransition','FlipAngular3DTransition',

            'ShuffleTransition',
            'TurnOffTilesTransition',
            'FadeTRTransition', 'FadeBLTransition',
            'FadeUpTransition', 'FadeDownTransition',

            'ShrinkGrowTransition',
            'CornerMoveTransition',
            'EnvelopeTransition',

            'SplitRowsTransition', 'SplitColsTransition',

            'FadeTransition',

            'ZoomTransition',
            ]

class TransitionScene(scene.Scene):
    """TransitionScene
    A Scene that takes two scenes and makes a transition between them.
    
    These scenes are children of the transition scene.
    """
    
    def __init__(self, dst, duration=1.25, src=None):
        '''Initializes the transition

        :Parameters:
            `dst` : Scene
                Incoming scene
            `duration` : float
                Duration of the transition in seconds. Default: 1.25
            `src` : Scene
                Outgoing scene. Default: current scene
        '''
        super(TransitionScene, self).__init__()

        self.in_scene = dst                     #: scene that will replace the old one
        if src == None:
            src = director.scene
        self.out_scene = src                    #: scene that will be replaced
        self.duration = duration                #: duration in seconds of the transition
        if not self.duration:
            self.duration = 1.25

        if self.out_scene is None:
            raise Exception("You need to specfy a `src` argument")

        if id(self.out_scene) == id(self.in_scene):
            raise Exception("Incoming scene must be different from outgoing scene")

        self.start()
        
    def start(self):
        '''Adds the incoming scene with z=1 and the outgoing scene with z=0'''
        self.add( self.in_scene, z=1 )
        self.add( self.out_scene, z=0 )

    def finish(self):
        '''Called when the time is over.
        It removes both the incoming and the outgoing scenes from the transition scene,
        and restores the outgoing scene's attributes like: position, visible and scale.
        '''
        self.remove( self.in_scene )
        self.remove( self.out_scene )
        self.restore_out()
        director.replace( self.in_scene )

    def hide_out_show_in( self ):
        '''Hides the outgoing scene and shows the incoming scene'''
        self.in_scene.visible = True
        self.out_scene.visible = False

    def hide_all( self ):
        '''Hides both the incoming and outgoing scenes'''
        self.in_scene.visible = False
        self.out_scene.visible = False

    def restore_out( self ):
        '''Restore the position, visible and scale attributes of the outgoing scene
        to the original values'''
        self.out_scene.visible = True
        self.out_scene.position = (0,0)
        self.out_scene.scale = 1
        
class RotoZoomTransition(TransitionScene):
    '''Rotate and zoom out the outgoing scene, and then rotate and zoom in the incoming 
    '''

    def __init__( self, *args, **kwargs ):
        super(RotoZoomTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        self.in_scene.scale = 0.001
        self.out_scene.scale = 1.0

        self.in_scene.transform_anchor = (width // 2, height //2 )
        self.out_scene.transform_anchor = (width // 2, height //2 )

        rotozoom = ( ScaleBy(0.001, duration=self.duration/2.0 ) | \
                     Rotate(360 * 2, duration=self.duration/2.0 ) ) + \
                Delay( self.duration / 2.0 )

        self.out_scene.do( rotozoom ) 
        self.in_scene.do( Reverse(rotozoom) + CallFunc(self.finish) )

class JumpZoomTransition(TransitionScene):
    '''Zoom out and jump the outgoing scene, and then jump and zoom in the incoming 
    '''

    def __init__( self, *args, **kwargs ):
        super(JumpZoomTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        self.in_scene.scale = 0.5
        self.in_scene.position = ( width, 0 )
        self.in_scene.transform_anchor = (width // 2, height //2 )
        self.out_scene.transform_anchor = (width // 2, height //2 )

        jump = JumpBy( (-width,0), width//4, 2, duration=self.duration / 4.0 )
        scalein = ScaleTo( 1, duration=self.duration / 4.0 )
        scaleout = ScaleTo( 0.5, duration=self.duration / 4.0 )

        jumpzoomout = scaleout + jump
        jumpzoomin = jump + scalein

        delay = Delay( self.duration / 2.0 )
        self.out_scene.do(  jumpzoomout ) 
        self.in_scene.do( delay + jumpzoomin + CallFunc(self.finish) )


class MoveInLTransition(TransitionScene):
    '''Move in from to the left the incoming scene.
    '''
    def __init__( self, *args, **kwargs ):
        super(MoveInLTransition, self ).__init__( *args, **kwargs)

        self.init()
        a = self.get_action()

        self.in_scene.do( (Accelerate(a,0.5) ) + CallFunc(self.finish) )

    def init(self):
        width, height = director.get_window_size()
        self.in_scene.position=(-width,0)

    def get_action(self):
        return MoveTo( (0,0), duration=self.duration)


class MoveInRTransition(MoveInLTransition):
    '''Move in from to the right the incoming scene.
    '''
    def init(self):
        width, height = director.get_window_size()
        self.in_scene.position=(width,0)

    def get_action(self):
        return MoveTo( (0,0), duration=self.duration)


class MoveInTTransition(MoveInLTransition):
    '''Move in from to the top the incoming scene.
    '''
    def init(self):
        width, height = director.get_window_size()
        self.in_scene.position=(0,height)

    def get_action(self):
        return MoveTo( (0,0), duration=self.duration)


class MoveInBTransition(MoveInLTransition):
    '''Move in from to the bottom the incoming scene.
    '''
    def init(self):
        width, height = director.get_window_size()
        self.in_scene.position=(0,-height)

    def get_action(self):
        return MoveTo( (0,0), duration=self.duration)

class SlideInLTransition(TransitionScene):
    '''Slide in the incoming scene from the left border.
    '''
    def __init__( self, *args, **kwargs ):
        super(SlideInLTransition, self ).__init__( *args, **kwargs)

        self.width, self.height = director.get_window_size()
        self.init()

        move = self.get_action()

        self.in_scene.do( Accelerate(move,0.5) )
        self.out_scene.do( Accelerate(move,0.5) + CallFunc( self.finish) )

    def init(self):
        self.in_scene.position=( -self.width,0)

    def get_action(self):
        return MoveBy( (self.width,0), duration=self.duration)


class SlideInRTransition(SlideInLTransition):
    '''Slide in the incoming scene from the right border.
    '''
    def init(self):
        self.in_scene.position=(self.width,0)

    def get_action(self):
        return MoveBy( (-self.width,0), duration=self.duration)


class SlideInTTransition(SlideInLTransition):
    '''Slide in the incoming scene from the top border.
    '''
    def init(self):
        self.in_scene.position=(0,self.height)

    def get_action(self):
        return MoveBy( (0,-self.height), duration=self.duration)


class SlideInBTransition(SlideInLTransition):
    '''Slide in the incoming scene from the bottom border.
    '''
    def init(self):
        self.in_scene.position=(0,-self.height)

    def get_action(self):
        return MoveBy( (0,self.height), duration=self.duration)


class FlipX3DTransition(TransitionScene):
    '''Flips the screen horizontally.
    The front face is the outgoing scene and the back face is the incoming scene.
    '''
    def __init__( self, *args, **kwargs ):
        super(FlipX3DTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        turnongrid = Waves3D( amplitude=0, duration=0, grid=(1,1), waves=2 )
        flip90 = OrbitCamera( angle_x=0,  delta_z=90, duration = self.duration / 2.0 )
        flipback90 = OrbitCamera( angle_x=0, angle_z=90, delta_z=90, duration = self.duration / 2.0 )

        self.in_scene.visible = False
        flip = turnongrid + \
                flip90 + \
                CallFunc(self.hide_all) + \
                FlipX3D(duration=0) + \
                CallFunc( self.hide_out_show_in ) + \
                flipback90
        
        self.do( flip + \
                CallFunc(self.finish) + \
                StopGrid() )


class FlipY3DTransition(TransitionScene):
    '''Flips the screen vertically.
    The front face is the outgoing scene and the back face is the incoming scene.
    '''
    def __init__( self, *args, **kwargs ):
        super(FlipY3DTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        turnongrid = Waves3D( amplitude=0, duration=0, grid=(1,1), waves=2 )
        flip90 = OrbitCamera( angle_x=90, delta_z=-90, duration = self.duration / 2.0 )
        flipback90 = OrbitCamera( angle_x=90, angle_z=90, delta_z=90, duration = self.duration / 2.0 )

        self.in_scene.visible = False
        flip = turnongrid + \
                flip90 + \
                CallFunc(self.hide_all) + \
                FlipX3D(duration=0) + \
                CallFunc( self.hide_out_show_in ) + \
                flipback90
        
        self.do( flip + \
                    CallFunc(self.finish) + \
                    StopGrid() )

class FlipAngular3DTransition(TransitionScene):
    '''Flips the screen half horizontally and half vertically.
    The front face is the outgoing scene and the back face is the incoming scene.
    '''
    def __init__( self, *args, **kwargs ):
        super(FlipAngular3DTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        turnongrid = Waves3D( amplitude=0, duration=0, grid=(1,1), waves=2 )
        flip90 = OrbitCamera( angle_x=45,  delta_z=90, duration = self.duration / 2.0 )
        flipback90 = OrbitCamera( angle_x=45, angle_z=90, delta_z=90, duration = self.duration / 2.0 )

        self.in_scene.visible = False
        flip = turnongrid + \
                flip90 + \
                CallFunc(self.hide_all) + \
                FlipX3D(duration=0) + \
                CallFunc( self.hide_out_show_in ) + \
                flipback90
        
        self.do( flip + \
                    CallFunc(self.finish) + \
                    StopGrid() )


class ShuffleTransition(TransitionScene):
    '''Shuffle the outgoing scene, and then reorder the tiles with the incoming scene.
    '''
    def __init__( self, *args, **kwargs ):
        super(ShuffleTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()
        aspect = width / float(height)
        x,y = int(12*aspect), 12

        shuffle = ShuffleTiles( grid=(x,y), duration=self.duration/2.0, seed=15 )
        self.in_scene.visible = False

        self.do( shuffle + \
                 CallFunc(self.hide_out_show_in) + \
                 Reverse(shuffle) + \
                 CallFunc(self.finish) + \
                 StopGrid()
                )


class ShrinkGrowTransition(TransitionScene):
    '''Shrink the outgoing scene while grow the incoming scene
    '''
    def __init__( self, *args, **kwargs ):
        super(ShrinkGrowTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        self.in_scene.scale = 0.001
        self.out_scene.scale = 1

        self.in_scene.transform_anchor = ( 2*width / 3.0, height / 2.0 )
        self.out_scene.transform_anchor = ( width / 3.0, height / 2.0 )

        scale_out = ScaleTo( 0.01, duration=self.duration )
        scale_in = ScaleTo( 1.0, duration=self.duration )

        self.in_scene.do( Accelerate(scale_in,0.5) )
        self.out_scene.do( Accelerate(scale_out,0.5) + CallFunc( self.finish) )


class CornerMoveTransition(TransitionScene):
    '''Moves the bottom-right corner of the incoming scene to the top-left corner
    '''
    def __init__( self, *args, **kwargs ):
        super(CornerMoveTransition, self ).__init__( *args, **kwargs)

        self.out_scene.do( MoveCornerUp( duration=self.duration ) + \
                            CallFunc(self.finish) + \
                            StopGrid() )

    def start(self):
        # don't call super. overriding order
        self.add( self.in_scene, z=0 )
        self.add( self.out_scene, z=1 )

class EnvelopeTransition(TransitionScene):
    '''From the outgoing scene:
        - moves the top-right corner to the center
        - moves the bottom-left corner to the center

      From the incoming scene:
        - performs the reverse action of the outgoing scene
    '''
    def __init__( self, *args, **kwargs ):
        super(EnvelopeTransition, self ).__init__( *args, **kwargs)

        self.in_scene.visible = False

        move = QuadMoveBy( delta0=(320,240), delta1=(-630,0), delta2=(-320,-240), delta3=(630,0), duration=self.duration / 2.0 )
#        move = Accelerate( move )
        self.do( move + \
                 CallFunc(self.hide_out_show_in) + \
                 Reverse(move) + \
                 CallFunc(self.finish) + \
                 StopGrid()
                 )


class FadeTRTransition(TransitionScene):
    '''Fade the tiles of the outgoing scene from the left-bottom corner the to top-right corner.
    '''
    def __init__( self, *args, **kwargs ):
        super(FadeTRTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()
        aspect = width / float(height)
        x,y = int(12*aspect), 12

        a = self.get_action(x,y)

#        a = Accelerate( a)
        self.out_scene.do( a + \
                            CallFunc(self.finish) + \
                            StopGrid() )

    def start(self):
        # don't call super. overriding order
        self.add( self.in_scene, z=0 )
        self.add( self.out_scene, z=1 )

    def get_action(self,x,y):
        return FadeOutTRTiles( grid=(x,y), duration=self.duration )

class FadeBLTransition(FadeTRTransition):
    '''Fade the tiles of the outgoing scene from the top-right corner to the bottom-left corner.
    '''
    def get_action(self,x,y):
        return FadeOutBLTiles( grid=(x,y), duration=self.duration )

class FadeUpTransition(FadeTRTransition):
    '''Fade the tiles of the outgoing scene from the bottom to the top.
    '''
    def get_action(self,x,y):
        return FadeOutUpTiles( grid=(x,y), duration=self.duration )

class FadeDownTransition(FadeTRTransition):
    '''Fade the tiles of the outgoing scene from the top to the bottom.
    '''
    def get_action(self,x,y):
        return FadeOutDownTiles( grid=(x,y), duration=self.duration )

class TurnOffTilesTransition(TransitionScene):
    '''Turn off the tiles of the outgoing scene in random order
    '''
    def __init__( self, *args, **kwargs ):
        super(TurnOffTilesTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()
        aspect = width / float(height)
        x,y = int(12*aspect), 12

        a = TurnOffTiles( grid=(x,y), duration=self.duration )
#        a = Accelerate( a)
        self.out_scene.do( a + \
                        CallFunc(self.finish) + \
                        StopGrid() )

    def start(self):
        # don't call super. overriding order
        self.add( self.in_scene, z=0 )
        self.add( self.out_scene, z=1 )


class FadeTransition(TransitionScene):
    '''Fade out the outgoing scene and then fade in the incoming scene.'''
    def __init__( self, *args, **kwargs ):
        super(FadeTransition, self ).__init__( *args, **kwargs)

        self.fadelayer = ColorLayer(0,0,0,0)

        self.in_scene.visible = False
        self.add( self.fadelayer, z=2 )


    def on_enter( self ):
        super( FadeTransition, self).on_enter()
        self.fadelayer.do( FadeIn( duration=self.duration/2.0) + \
                           CallFunc( self.hide_out_show_in) + \
                           FadeOut( duration=self.duration /2.0 ) + \
                           CallFunc( self.finish) )
    def on_exit( self ):
        super( FadeTransition, self).on_exit()
        self.remove( self.fadelayer )

class SplitColsTransition(TransitionScene):
    '''Splits the screen in columns.
    The odd columns goes upwards while the even columns goes downwards.
    '''
    def __init__( self, *args, **kwargs ):
        super(SplitColsTransition, self ).__init__( *args, **kwargs)

        width, height = director.get_window_size()

        self.in_scene.visible = False
        flip_a =  self.get_action()
        flip = flip_a + \
                CallFunc( self.hide_out_show_in ) + \
                Reverse(flip_a)
        
        self.do( AccelDeccel(flip) + \
                    CallFunc(self.finish) + \
                    StopGrid() )

    def get_action( self ):
        return SplitCols( cols=3, duration=self.duration/2.0)

class SplitRowsTransition(SplitColsTransition):
    '''Splits the screen in rows.
    The odd rows goes to the left while the even rows goes to the right.
    '''
    def get_action( self ):
        return SplitRows( rows=3, duration=self.duration/2.0)


class ZoomTransition(TransitionScene):
    '''Zoom and FadeOut the outgoing scene.'''

    def __init__(self, *args, **kwargs):
        if 'src' in kwargs or len(args) == 3:
            raise Exception("ZoomTransition does not accept 'src' parameter.")

        super(ZoomTransition, self ).__init__( *args, **kwargs)
        self.out_scene.visit()

    def start(self):
        screensprite = self._create_out_screenshot()

        zoom = ScaleBy(2, self.duration) | FadeOut(self.duration)
        screensprite.do(zoom)

        restore = CallFunc(self.finish)
        self.in_scene.do(Delay(self.duration * 2) + restore)

        self.add(screensprite, z=1)
        self.add(self.in_scene, z=0)

    def finish(self):
        '''Called when the time is over.
        It removes both the incoming and the outgoing scenes from the transition scene,
        and restores the outgoing scene's attributes like: position, visible and scale.
        '''
        self.remove( self.in_scene )
        self.restore_out()
        director.replace( self.in_scene )

    def _create_out_screenshot(self):
        # TODO: try to use `pyglet.image.get_buffer_manager().get_color_buffer()`
        #       instead of create a new BufferManager... note that pyglet uses
        #       a BufferManager singleton that fail when you change the window
        #       size.
        buffer = pyglet.image.BufferManager()
        image = buffer.get_color_buffer()

        width, height = director.window.width, director.window.height 
        actual_width, actual_height = director.get_window_size()

        out = Sprite(image)
        out.position = actual_width / 2, actual_height / 2
        out.scale = max(actual_width / float(width), actual_height / float(height))

        return out



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos.actions import *

import pyglet
from pyglet.gl import *


class BackgroundLayer( cocos.layer.Layer ):
    def __init__(self):
        super( BackgroundLayer, self ).__init__()
        self.img = pyglet.resource.image('background_image.png')

    def draw( self ):
        glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        glPopMatrix()

class TestLayer(cocos.layer.Layer):
    def __init__(self):
        super( TestLayer, self ).__init__()
        
        x,y = director.get_window_size()
        
        self.sprite = Sprite( 'grossini.png', (x/2,y/2), scale = 1 )
        self.add( self.sprite )
        self.sprite.do( Repeat( ScaleBy( 5, 2 ) + ScaleBy( 0.2, 2 )  ) )
        self.sprite.do( Repeat( RotateBy( 360, 10 ) ) )

        self.sprite.do( Waves( duration=5 ) + \
                            Twirl( amplitude=1, twirls=3, grid=(32,24), duration=5 ) + \
                            WavesTiles3D( waves=4, grid=(32,24), duration=5 ) + \
                            TurnOffTiles( grid=(32,24), duration=2) + \
                            Reverse( TurnOffTiles( grid=(32,24), duration=2) ) + \
                            StopGrid() )
        

if __name__ == "__main__":
    director.init()
    background = BackgroundLayer()
    test_layer = TestLayer ()
    main_scene = cocos.scene.Scene ()

    main_scene.add( background, z=0 )
    main_scene.add( test_layer, z=1 )

    director.run (main_scene)



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#

import pyglet
import cocos
from cocos.director import director
from cocos.actions import *
from cocos.layer import *



class BackgroundLayer( cocos.layer.Layer ):
    def __init__(self):
        super( BackgroundLayer, self ).__init__()
        self.img = pyglet.resource.image('background_image.png')

    def draw( self ):
        self.img.blit(0,0)

def toggle_fullscreen():
    director.window.set_fullscreen( not director.window.fullscreen )

if __name__ == "__main__":
    director.init( resizable=True )
    director.set_depth_test()

    main_scene = cocos.scene.Scene()

    main_scene.add( BackgroundLayer(), z=0 )

    # set a 3d grid with a grid3d action
    e = WavesTiles3D( amplitude=60, waves=2, grid=(32,24), duration=3)
    f = ShuffleTiles( duration=3, grid=(32,24) )

    main_scene.do( e +  \
        CallFunc( toggle_fullscreen ) + \
        Reverse(e) + \
        CallFunc(toggle_fullscreen) + \
        f + \
        CallFunc(toggle_fullscreen) + \
        Reverse(f) + \
        StopGrid() \
        )

    director.run (main_scene)



# ----------------------------------------------------------------------------
# cocos2d
# Copyright (c) 2008 Daniel Moisset, Ricardo Quesada, Rayentray Tappa, Lucio Torre
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of cocos2d nor the names of its
#     contributors may be used to endorse or promote products
#     derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Grid Actions

Grid Actions
============

There are 2 kinds of grids:

  - `Grid3D` : A 3D grid with x,y and z coordinates
  - `TiledGrid3D` : A 3D grid with x,y and z coordinates, composed
     with independent tiles


Hence, there are 2 kinds of grid actions:

  - `Grid3DAction`
  - `TiledGrid3DAction`

The `Grid3DAction` can modify any of vertex of the grid in any direction (x,y or z).
The `TiledGrid3DAction` can modify any tile of the grid without modifying the adjacent tiles.

To understand visually the difference between these 2 kinds of grids, try these examples:

  - run ``test/test_shakytiles3d.py`` to see a `TiledGrid3DAction` example 
  - run ``test/test_shaky3d.py`` to see the `Grid3DAction` counterpart
'''

__docformat__ = 'restructuredtext'

from pyglet.gl import *

from cocos.grid import Grid3D, TiledGrid3D
from cocos.director import director
from cocos.euclid import *
from base_actions import *

__all__ = [ 'GridException',            # Grid Exceptions
            'GridBaseAction',               # Base classes
            'Grid3DAction',
            'TiledGrid3DAction',
            
            'AccelAmplitude',           # Amplitude modifiers
            'DeccelAmplitude',
            'AccelDeccelAmplitude',
            
            'StopGrid',
            'ReuseGrid',
            ]

class GridException( Exception ):
    pass
    
class GridBaseAction( IntervalAction ):
    '''GridBaseAction is the base class of all Grid Actions.'''
    def init( self, grid=(4,4), duration=5):
        """Initialize the Grid Action

        :Parameters:
            `grid` : (int,int)
                Number of horizontal and vertical quads in the grid
            `duration` : int 
                Number of seconds that the action will last
        """
        self.duration = duration
        if not isinstance(grid,Point2):
            grid = Point2( *grid)
        self.grid = grid

    def start( self ):        
        new_grid = self.get_grid()

        if self.target.grid and self.target.grid.reuse_grid > 0:                
            # Reusing the grid 
            if self.target.grid.active \
                    and self.grid == self.target.grid.grid \
                    and type(new_grid) == type(self.target.grid):
                # since we are reusing the grid,
                # we must "cheat" the action that the original vertex coords are
                # the ones that were inhereted.
                self.target.grid.vertex_points = self.target.grid.vertex_list.vertices[:]
                self.target.grid.reuse_grid -= 1
                self.target.grid.reuse_grid = max(0, self.target.grid.reuse_grid)
            else:
                # condition are not met
                raise GridException("Cannot reuse grid. class grid or grid size did not match: %s vs %s and %s vs %s"
                                    % ( str(self.grid), str(self.target.grid.grid), type(new_grid), type(self.target.grid) ) )
        else:
            # Not reusing the grid
            if self.target.grid and self.target.grid.active:
                self.target.grid.active = False
            self.target.grid = new_grid
            self.target.grid.init( self.grid )
            self.target.grid.active = True

        x,y = director.get_window_size()
        self.size_x = x // self.grid.x
        self.size_y = y // self.grid.y
      
    def __reversed__(self):
        return _ReverseTime(self)
 
 
class Grid3DAction( GridBaseAction ):
    '''Action that does transformations
    to a 3D grid ( `Grid3D` )'''

    def get_grid(self):
        return Grid3D()
    
    def get_vertex( self, x, y):
        '''Get the current vertex coordinate

        :Parameters:
            `x` : int 
               x-vertex
            `y` : int
               y-vertex

        :rtype: (float, float, float)
        '''
        return self.target.grid.get_vertex(x,y)

    def get_original_vertex( self, x, y):
        '''Get the original vertex coordinate.
        The original vertices are the ones weren't modified by the current action.

        :Parameters:
            `x` : int 
               x-vertex
            `y` : int
               y-vertex

        :rtype: (float, float, float)
        '''
        return self.target.grid.get_original_vertex(x,y)


    def set_vertex( self, x, y, v):
        '''Set a vertex point is a certain value

        :Parameters:
            `x` : int 
               x-vertex
            `y` : int
               y-vertex
            `v` : (float, float, float)
                tuple value for the vertex
        '''
        return self.target.grid.set_vertex(x,y,v)


class TiledGrid3DAction( GridBaseAction ):
    '''Action that does transformations
    to a grid composed of tiles ( `TiledGrid3D` ).
    You can transform each tile individually'''

    def get_grid(self):
        return TiledGrid3D()
    
    def set_tile(self, x, y, coords):
        '''Set the 4 tile coordinates

        Coordinates positions::

            3 <-- 2
                  ^
                  |
            0 --> 1

        :Parameters:
            `x` : int 
                x coodinate of the tile
            `y` : int 
                y coordinate of the tile
            `coords` : [ float, float, float, float, float, float, float, float, float, float, float, float ]
                The 4 coordinates in the format (x0, y0, z0, x1, y1, z1,..., x3, y3, z3)
        '''
        return self.target.grid.set_tile(x,y,coords)
    
    def get_original_tile(self, x, y):
        '''Get the 4-original tile coordinates.

        Coordinates positions::

            3 <-- 2
                  ^
                  |
            0 --> 1

        :Parameters:
            `x` : int
                x coordinate of the tile
            `y` : int
                y coordinate of the tile

        :rtype: [ float, float, float, float, float, float, float, float, float, float, float, float ]
        :returns: The 4 coordinates with the following order: x0, y0, z0, x1, y1, z1,...,x3, y3, z3
        '''
        return self.target.grid.get_original_tile(x,y)

    def get_tile(self, x, y):
        '''Get the current tile coordinates.

        Coordinates positions::

            3 <-- 2
                  ^
                  |
            0 --> 1

        :Parameters:
            `x` : int
                x coordinate of the tile
            `y` : int
                y coordinate of the tile

        :rtype: [ float, float, float, float, float, float, float, float, float, float, float, float ]
        :returns: The 4 coordinates with the following order: x0, y0, z0, x1, y1, z1,...,x3, y3, z3
        '''
        return self.target.grid.get_tile(x,y)
        

class AccelDeccelAmplitude( IntervalAction ):
    """
    Increases and Decreases the amplitude of Wave
    
    Example::

        # when t=0 and t=1 the amplitude will be 0
        # when t=0.5 (half time), the amplitude will be 40
        action = AccellDeccelAmplitude( Wave3D( waves=4, amplitude=40, duration=6) )
        scene.do( action )
    """
    def init(self, other, rate=1.0 ):
        """Init method.

        :Parameters:
            `other` : `IntervalAction`
                The action that will be affected
            `rate` : float
                The acceleration rate. 1 is linear (default value)
        """
        if not hasattr(other,'amplitude'):
            raise GridAction("Invalid Composition: IncAmplitude needs an action with amplitude")

        self.other = other
        self.rate = rate
        self.duration = other.duration

    def start(self):
        self.other.target = self.target
        self.other.start()

    def update(self, t):
        f = t*2
        if f > 1:
            f -= 1
            f = 1 -f  

        self.other.amplitude_rate = f**self.rate
        self.other.update( t )

    def __reversed__(self):
        return AccelDeccelAmplitude( Reverse(self.other) )

class AccelAmplitude( IntervalAction ):
    """
    Increases the waves amplitude from 0 to self.amplitude
    
    Example::

        # when t=0 the amplitude will be 0
        # when t=1 the amplitude will be 40
        action = AccelAmplitude( Wave3D( waves=4, amplitude=40, duration=6), rate=1.0 )
        scene.do( action )
    """
    def init(self, other, rate=1 ):
        """Init method.

        :Parameters:
            `other` : `IntervalAction`
                The action that will be affected
            `rate` : float
                The acceleration rate. 1 is linear (default value)
        """
        if not hasattr(other,'amplitude'):
            raise GridAction("Invalid Composition: IncAmplitude needs an action with amplitude")
        
        self.other = other
        self.duration = other.duration
        self.rate = rate

    def start(self):
        self.other.target = self.target
        self.other.start()

    def update(self, t):
        self.other.amplitude_rate = (t ** self.rate)
        self.other.update( t )

    def __reversed__(self):
        return DeccelAmplitude( Reverse(self.other), rate=self.rate )


class DeccelAmplitude( AccelAmplitude ):
    """
    Decreases the waves amplitude from self.amplitude to 0
    
    Example::

        # when t=1 the amplitude will be 0
        # when t=0 the amplitude will be 40
        action = DeccelAmplitude( Wave3D( waves=4, amplitude=40, duration=6), rate=1.0 )
        scene.do( action )
    """

    def update(self, t):
        self.other.amplitude_rate = ((1-t) ** self.rate)
        self.other.update( t )

    def __reversed__(self):
        return AccelAmplitude( Reverse(self.other), rate=self.rate )


class StopGrid( InstantAction ):
    """StopGrid disables the current grid.
    Every grid action, after finishing, leaves the screen with a certain grid figure.
    This figure will be displayed until StopGrid or another Grid action is executed.
    
    Example::
    
        scene.do( Waves3D( duration=2) + StopGrid() )
    """
    def update(self, t):
        if self.target.grid and self.target.grid.active:
            self.target.grid.active = False

class ReuseGrid( InstantAction ):
    """Will reuse the current grid for the next grid action.
    The next grid action must have these properties:
    
        - Be of the same class as the current one ( `Grid3D` or `TiledGrid3D` )
        - Have the same size
    
    If these condition are met, then the next grid action will receive as the ``original vertex``
    or ``original tiles`` the current ones.
    
    Example::
    
        scene.do( Waves3D( duration=2) + ReuseGrid() + Lens3D(duration=2) )
    """
    def init(self, reuse_times=1):
        '''
        :Parameters:
            `reuse_times` : int
                Number of times that the current grid will be reused by Grid actions. Default: 1
        '''
        self.reuse_times = reuse_times
    def start(self):
        if self.target.grid and self.target.grid.active:
            self.target.grid.reuse_grid += self.reuse_times
        else:
            raise GridException("ReuseGrid must be used when a grid is still active")



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.sprite import Sprite
import pyglet


if __name__ == "__main__":
    director.init()
    x,y = director.get_window_size()

    main_scene = (
        cocos.scene.Scene()
            .add( cocos.layer.ColorLayer( 255,255,0,255) )
            .add( cocos.layer.Layer() 
                .add( Sprite('grossini.png', (x/2, y/2))
            )
        )
    )
    director.run (main_scene)



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.actions import  *
from cocos.sprite import Sprite
import pyglet

class TestLayer(cocos.layer.Layer):

    def _step( self, dt ):
        super(TestLayer,self)._step(dt)
        print 'shall not happen'
        print self.rotation
         

if __name__ == "__main__":
    director.init()
    test_layer = TestLayer ()
    main_scene = cocos.scene.Scene()
    test_layer.do( RotateBy(360, duration=2) )
    director.run (main_scene)



import cocosnode
import pyglet
from pyglet.gl import *
from euclid import *
import math
import copy

cuadric_t = '''
void main() {
    vec2 pos = gl_TexCoord[0].st;
    float res = pos.x*pos.x - pos.y;
    if (res<0.0) {
        gl_FragColor = gl_Color;
    } else {
        gl_FragColor = vec4(0.0,0.0,0.0,0.0);
    }
}
'''

class Shader(object):
    def __init__(self, source):
        self.source = source 
        self.shader_no = glCreateShader(self.shader_type)
        if not self.shader_no:
            raise Exception("could not create shader")
        prog = (c_char_p * 1)(source+chr(0))
        length = (c_int * 1)(0)
        glShaderSource(self.shader_no, 1,
                          cast(prog, POINTER(POINTER(c_char))),
                          cast(0, POINTER(c_int)))
        glCompileShader(self.shader_no)
        self.program_no = glCreateProgram()
        if not self.program_no:
            raise Exception("could not create program")
        glAttachShader(self.program_no, self.shader_no)
        glLinkProgram(self.program_no)
        
    def begin(self):
        glUseProgram(self.program_no)
    def end(self):
        glUseProgram(0)
    

class VertexShader(Shader):
    shader_type = GL_VERTEX_SHADER

class FragmentShader(Shader):
    shader_type = GL_FRAGMENT_SHADER
    
#cuadric = FragmentShader(cuadric_t)

def parameter(default=None):
    o = object()
    name = str(id(o))
    
    def setter(self, value):
        self._dirty = True
        setattr(self, "_"+name, value)
        
    def getter(self):
        return getattr(self, "_"+name, default)
    
    
    return property(getter, setter)
    
ROUND_CAP, SQUARE_CAP, BUTT_CAP = range(3)
MITER_JOIN, BEVEL_JOIN, ROUND_JOIN = range(3)

class Context(object):
    def __init__(self):
        self.color = 255,255,255,255
        self.stroke_width = 2
        self.cap = ROUND_CAP
        self.join = ROUND_JOIN
        self.transform = Matrix3()
        
    def set_state(self):
        glPushAttrib(GL_CURRENT_BIT|GL_LINE_BIT)
        glColor4ub(*self.color)
        glLineWidth(self.stroke_width)
        
    def unset_state(self):
        glPopAttrib()
        
    def copy(self):
        return copy.deepcopy(self)
        
        
def flatten(*args):
    ret = []
    for a in args:
        for v in a:
            ret.append( v )
    return ret
    
class Segment:
    def __init__(self, start, end, width):
        self.start = Point2(*start)
        self.end = Point2(*end)
        self.width = width
        self._tl = None
        self._bl = None
        self._tr = None
        self._br = None
        
    @property
    def direction(self):
        return Vector2( *(self.end-self.start)).normalized()
        
    @property
    def line_width(self):
        return ( 
            Matrix3.new_rotate(math.radians(90)) * self.direction *
            (self.width / 2.0)
            )
            
    @property
    def tl(self):
        if self._tl: return self._tl
        return self.end + self.line_width
    @property
    def tr(self):
        if self._tr: return self._tr
        return self.end - self.line_width
    @property
    def bl(self):
        if self._bl: return self._bl
        return self.start + self.line_width
    @property
    def br(self):
        if self._br: return self._br
        return self.start - self.line_width   
        
    @property
    def left(self):
        return LineSegment2( Point2(*self.bl), Point2(*self.tl) )
        
    @property
    def right(self):
        return LineSegment2( Point2(*self.br), Point2(*self.tr) )
        
    @property
    def points(self):
        return flatten( self.bl, self.br, self.tr, self.bl, self.tr, self.tl )
        
    def reversed(self):
        return Segment(self.end, self.start, self.width)
        
class Canvas(cocosnode.CocosNode):
    def __init__(self):
        super(Canvas, self).__init__()
        self._dirty = True
        self._color = 255,255,255,255
        self._stroke_width = 1
        self._parts = []        
        self._vertex_list = None
        self._context = Context()
        self._context_stack = []
        self._texture = image = pyglet.resource.image('draw_texture.png').get_texture()
        
        self._context_change = True
        self._position = 0,0
        
    def draw(self):
        if self._dirty:
            self._context = Context()
            self._parts = []
            self.free()
            self.render()
            self.build_vbo()
            self._dirty = False
            
        # set
        glEnable(self._texture.target)
        glBindTexture(self._texture.target, self._texture.id)
        glPushAttrib(GL_COLOR_BUFFER_BIT)
        
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        glPushMatrix()
        self.transform()
        #cuadric.begin()
        self._vertex_list.draw(GL_TRIANGLES)
        #cuadric.end()
        
        # unset
        glPopMatrix()
        glPopAttrib()
        glDisable(self._texture.target)
        
    def endcap(self, line, cap_type):
        strip = []
        texcoord = []
        
        if cap_type == ROUND_CAP:
            s = Segment( line.start, 
                        line.start + (-line.direction) * line.width / 2,
                        line.width
                        )
            strip.extend([int(x) for x in flatten(
                            s.bl, s.br, s.end,
                            s.br, s.tr, s.end,
                            s.bl, s.tl, s.end
                            )])
            texcoord.extend([
                    0.1,0.9,0.1,0.5,0.5,0.9,
                    0,0,0.5,0,1,1,
                    0,0,0.5,0,1,1,
                    ])
        elif cap_type == SQUARE_CAP:
            segment = Segment( line.start, 
                        line.start + (-line.direction) * line.width / 2,
                        line.width
                        )
            strip.extend([int(x) for x in segment.points])
            texcoord.extend( flatten(*[ (0.1,0.9,0.1,0.5,0.5,0.9) for x in range(len(segment.points)/6) ]) ) 
            
        return strip, texcoord
        
    def build_vbo(self):
        strip = []
        colors = []
        texcoord = []
        for ctx, parts in self._parts:
            start_len = len(strip)
            for line in parts:
            
                # build the line segments
                last = line[0]
                segments = []
                for next in line[1:]:
                    segments.append( Segment( last, next, ctx.stroke_width ) )
                    last = next
                
                # do we need caps?
                if line[0] == line[-1]:
                    closed_path = True
                else:
                    closed_path = False
                    
                # add caps
                if not closed_path:
                    vertex, tex = self.endcap(segments[0], ctx.cap)
                    strip += vertex
                    texcoord += tex
                    vertex, tex = self.endcap(segments[-1].reversed(), ctx.cap)
                    strip += vertex
                    texcoord += tex                    
                 
                # update middle points  
                prev = None
                for i, current in enumerate(segments):
                    # if not starting line
                    if ( prev ):
                        # turns left
                        inter = prev.left.intersect( current.left )
                        if inter:
                            prev._tl = inter
                            current._bl = inter
                            bottom = prev.tr
                            top = current.br
                        else:
                            inter = prev.right.intersect( current.right )
                            if inter:
                                prev._tr = inter
                                current._br = inter
                                bottom = prev.tl
                                top = current.bl
                        
                    
                    # add elbow
                    if ( prev and inter ):
                            if ctx.join == BEVEL_JOIN:
                                strip.extend( [ int(x) for x in
                                    list(inter) + list(bottom) + list(top)
                                ])
                                texcoord += [ 0.1,0.9,0.1,0.5,0.5,0.9 ]
                            elif ctx.join in (MITER_JOIN, ROUND_JOIN):
                                far = Ray2(
                                    Point2(*bottom), prev.direction
                                    ).intersect(Ray2(
                                    Point2(*top), -current.direction
                                    ))
                                strip.extend( [ int(x) for x in
                                    list(inter) + list(bottom) + list(top) +
                                    list(bottom) + list(top) + list(far)
                                ])
                                if ctx.join == ROUND_JOIN:
                                    texcoord += [ 0.1,0.9,0.1,0.5,0.5,0.9, 0,0,1,1,0.5,0]
                                elif ctx.join == MITER_JOIN:
                                    texcoord += [ 0.1,0.9,0.1,0.5,0.5,0.9,0.1,0.9,0.1,0.5,0.5,0.9 ]

                    # rotate values
                    prev = current
                
                # add boxes for lines
                for s in segments:
                    strip.extend( [ int(x) for x in s.points ] )
                    texcoord += flatten(*[ (0.1,0.9,0.1,0.5,0.5,0.9)
                                for x in range( len(s.points)/6) 
                            ])
                            

            colors.extend( list(ctx.color)*((len(strip)-start_len)/2) )
            
        vertex_list = pyglet.graphics.vertex_list(len(strip)/2,
            ('v2i', strip),
            ('c4B', colors ),
            ('t2f', texcoord),
        )
        self._vertex_list = vertex_list
        
    def on_exit(self):
        super(Canvas, self).on_exit()
        self.free()
        
    def free(self):
        if self._vertex_list:
            self._vertex_list.delete()
            
    def set_color(self, color):
        self._context.color = color
        self._context_change = True
        
    def set_stroke_width(self, stroke_width):
        self._context.stroke_width = stroke_width
        self._context_change = True        
    
    def set_endcap(self, cap):
        self._context.cap = cap
        self._context_change = True   
        
    def set_join(self, join):
        self._context.join = join
        self._context_change = True 
        
    def rotate(self, radians):
        self._context.transform.rotate( radians )
    def translate(self, vect):
        self._context.transform.translate( *vect )
        
    def move_to(self, position):
        self._position = self._context.transform * Point2(*position)
        
    def line_to(self, end):
        if self._context_change:
            context, parts = self._context, [[self._position]]
            self._parts.append((context, parts))
            self._context = context.copy()
            self._context_change = False
        else:
            context, parts = self._parts[-1]

        end = self._context.transform * Point2(*end)
        
        if parts[-1][-1] == self._position:
            parts[-1].append( end )
        else:
            parts.append( [self._position, end] )
            
        self._position = end
        
    def push(self):
        self._context_stack.append( self._context.copy() )
        
    def pop(self):
        self._context = self._context_stack.pop()
    
        
class Line(Canvas):
    start = parameter()
    end = parameter()
    stroke_width = parameter()
    color = parameter()
    
    def __init__(self, start, end, color, stroke_width=1):
        super(Line, self).__init__()
        self.start = start
        self.end = end
        self.color = color
        self.stroke_width = stroke_width
        
    def render(self):
        self.set_color( self.color )
        self.set_stroke_width( self.stroke_width )
        self.move_to( self.start )
        self.line_to( self.end )
        
        
        


# ----------------------------------------------------------------------------
# cocos2d
# Copyright (c) 2008 Daniel Moisset, Ricardo Quesada, Rayentray Tappa, Lucio Torre
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of cocos2d nor the names of its
#     contributors may be used to endorse or promote products
#     derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
"""Batch

Batches
=======

Batches allow you to optimize the number of gl calls using pygllets batch

"""

__docformat__ = 'restructuredtext'

import cocosnode
from batch import *

import pyglet
from pyglet.graphics import OrderedGroup
from pyglet import image
from pyglet.gl import *

__all__ = ['BatchNode','BatchableNode','NodeGroup' ]


def ensure_batcheable(node):
    if not isinstance(node, BatchableNode):
        raise Exception("Children node of a batch must be have the batch mixin")
    for c in  node.get_children():
        ensure_batcheable(c)

class BatchNode( cocosnode.CocosNode ):
    def __init__(self):
        super(BatchNode, self).__init__()
        self.batch = pyglet.graphics.Batch()

    def add(self, child, z=0, name=None):
        ensure_batcheable(child)
        group = pyglet.graphics.OrderedGroup( z )
        child.set_batch( self.batch, group )

        super(BatchNode, self).add(child, z, name)

    def visit(self):
        self.batch.draw()

    def draw(self):
        pass#self.batch.draw()

class BatchableNode( cocosnode.CocosNode ):
    def add(self, child, z=0, name=None):
        batchnode = self.get_ancestor(BatchNode)
        if not batchnode:
            # this node was addded, but theres no batchnode in the
            # hierarchy. so we proceed as normal
            super(BatchableNode, self).add(child, z, name)
            return

        # we are being batched, so we set groups and batch
        # pre/same/post will be set, because if we have a
        # batchnode parent, we already executed set_batch on self
        ensure_batcheable(child)
        if z < 0:
            group = self.pre_group
        elif z == 0:
            group = self.same_group
        else:
            group = self.post_group

        super(BatchableNode, self).add(child, z, name)
        child.set_batch( self.batch, group )


    def remove(self, child):
        if isinstance(child, str):
            child = self.get(child)
        child.set_batch( None, None )
        super(BatchableNode, self).remove(child)


    def set_batch(self, batch, group):
        sprite_group = NodeGroup(self, group)
        self.pre_group = NodeGroup(self, OrderedGroup(-1, parent=group))
        self.group = OrderedGroup(0, parent=group)
        self.same_group = NodeGroup(self, self.group)
        self.post_group = NodeGroup(self, OrderedGroup(1, parent=group))
        self.batch = batch

        for z, child in self.children:
            if z < 0:
                group = self.pre_group
            elif z == 0:
                group = self.same_group
            else:
                group = self.post_group
            child.set_batch( self.batch, group )


class NodeGroup(pyglet.graphics.Group):
    def __init__(self, sprite, group):
        super(NodeGroup, self).__init__(parent=group)
        self.sprite = sprite

    def set_state(self):
        glPushMatrix()
        self.sprite.transform()

    def unset_state(self):
        glPopMatrix()



# ----------------------------------------------------------------------------
# cocos2d
# Copyright (c) 2008 Daniel Moisset, Ricardo Quesada, Rayentray Tappa, Lucio Torre
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#   * Neither the name of cocos2d nor the names of its
#     contributors may be used to endorse or promote products
#     derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Base actions

Base Actions
============

These actions are the 'mother' of all actions.
They are the building blocks.

'''

__docformat__ = 'restructuredtext'

import copy

__all__ = [ 
            'Action',                           # Base Class
            'IntervalAction', 'InstantAction',  # Important Base classes 
            'Sequence','Spawn','Repeat',        # Basic behaviors
            'Reverse','_ReverseTime',           # Reverse
            ]





class Action(object):
    '''Mother of all actions'''
    def __init__(self, *args, **kwargs):
        self.init(*args, **kwargs)
        self.target = None              #: `CocosNode` object that is the target of the action
        self._elapsed = None

    def init(self):
        """
        Gets called at initialization time, before a target is defined
        """
        pass

    def start(self):
        """
        Before we start executing an action, self.target is assigned and this method is called.
        It will be called for every execution of the action.
        """
        pass

    def stop(self):
        """
        After we finish executing an action this method is called.
        It will be called for every execution of the action.
        """
        pass
        
    def step(self, dt):
        """
        Gets called every frame. `dt` is the number of seconds that elapsed
        since the last call. If there was a pause and resume in the middle,
        the actual elapsed time may be bigger.

        This function will only be called by the `Layer`, but interval actions will
        be updated with the `IntervalAction.update` method.
        """
        if self._elapsed is None:
            self._elapsed = 0


        self._elapsed += dt

        if self.duration:
            self.update( min(1, self._elapsed/self.duration ) )
        else:
            self.update( 1 )
            self.stop()


    def __add__(self, action):
        """Is the Sequence Action"""
        return Sequence(self, action)

    def __mul__(self, other):
        if not isinstance(other, int):
            raise TypeError("Can only multiply actions by ints")
        if other <= 1:
            return self
        return  Loop(self, other)

    def __or__(self, action):
        """Is the Spawn Action"""
        return Spawn(self, action)

    def __reversed__(self):
        raise Exception("Action %s cannot be reversed"%(self.__class__.__name__))


class IntervalAction( Action ):
    """
    IntervalAction()

    Abstract Class that defines the direction of any Interval
    Action. Interval Actions are the ones that have a fixed duration,
    so we can make them go forward or backwards in time.

    For example: `MoveTo` , `MoveBy` , `RotateBy` are Interval Actions, while
    `Place`, `Show` and `CallFunc` aren't.

    Subclasses must ensure that instances have a duration attribute.
    """


    def update(self, t):
        """Gets called on every frame.
        `t` is in [0,1]
        If this action takes 5 seconds to execute, `t` will be equal to 0
        at 0 seconds. `t` will be 0.5 at 2.5 seconds and `t` will be 1 at 5sec.
        """
        pass


    def done(self):
        return self._elapsed >= self.duration

def Reverse( action ):
    """Reverses the behavior of the action

    Example::

        # rotates the sprite 180 degrees in 2 seconds counter clockwise
        action = Reverse( RotateBy( 180, 2 ) )
        sprite.do( action )
    """
    return action.__reversed__()

class InstantAction( Action ):
    """
    Instant actions are actions that happen just one call.
    """
    duration = 0

    def start(self):
        """
        Here we must do out stuff
        """
        pass

    def done(self):
        return True

    def update(self, t):
        pass
class Loop(IntervalAction):
    """Repeat one action for n times
    You can loop actions using:

        * the Loop() class
        * the overriden * operator

    Example::

        action = Loop( one, 10 )
        sprite.do( action )

        or:

        sprite.do( one * 10 )
    """
    def init(self,  one, times ):
        """Init method

        :Parameters:
            `one` : `Action`
                Action to be repeated
            `times` : int
                Number of times that the action will be repeated
        """

        self.one = one
        self.times = times

        if not hasattr(self.one, "duration"):
            raise Exception("You can only loop actions with finite duration, not repeats or others like that")
        
        self.duration = self.one.duration * times
        
        self.current = None
        self.last = None
        
    def start(self):
        self.duration = self.one.duration * self.times
        self.last = 0
        self.current_action = copy.deepcopy(self.one)
        self.current_action.target = self.target
        self.current_action.start()
        
    def __repr__(self):
        return "( %s * %i )" %( self.one, self.times )

    def update(self, t):
        current = int(t * float(self.times))
        new_t = (t - (current * (1./self.times) ) ) * self.times


        if current >= self.times: # we are done
            return
        # just more dt for the current action
        elif current == self.last:
            self.current_action.update(new_t)
        else:
            # finish the last action
            self.current_action.update(1)
            self.current_action.stop()
            
            for i in range(self.last+1, current):
                # fast forward the jumped actions            
                self.current_action = copy.deepcopy(self.one)
                self.current_action.target = self.target
                self.current_action.start()
                self.current_action.update(1)
                self.current_action.stop()
            
            # set new current action
            self.current_action = copy.deepcopy(self.one)
            self.current_action.target = self.target
            self.last = current
            
            # start a new action
            self.current_action.start()
            
            # feed dt
            self.current_action.update(new_t)
            
    def stop(self):
        self.current_action.update(1)
        self.current_action.stop()
        
    def __reversed__(self):
        return Loop( Reverse(self.one), self.times )
        
        
class Sequence(IntervalAction):
    """Run actions sequentially: One after another
    You can sequence actions using:

        * the Sequence() class
        * the overriden *+* operator

    Example::

        action = Sequence( one, Sequence( two, three) )
        sprite.do( action )

        or:

        sprite.do( one + two + three )
        """
    def init(self,  one, two, **kwargs ):
        """Init method

        :Parameters:
            `one` : `Action`
                The first action to execute
            `two` : `Action`
                The second action to execute
        """

        self.one = copy.deepcopy(one)
        self.two = copy.deepcopy(two)
        self.actions = [self.one, self.two]

        if not hasattr(self.one, "duration") or not hasattr(self.two, "duration"):
            raise Exception("You can only sequence actions with finite duration, not repeats or others like that")
        
        self.duration = self.one.duration + self.two.duration
        self.split = self.one.duration / float(self.duration)

        self.last = None

    def start(self):
        self.duration = self.one.duration + self.two.duration
        self.split = self.one.duration / float(self.duration)

        self.one.target = self.target
        self.two.target = self.target

    def __repr__(self):
        return "( %s + %s )" %( self.one, self.two )

    def update(self, t):
        start_t = 0
        found = None
        if t >= self.split:
            found = 1
            if self.split == 1:
                new_t = 1
            else:
                new_t = (t-self.split) / (1 - self.split )
        elif t < self.split:
            found = 0
            if self.split != 0:
                new_t = t / self.split
            else:
                new_t = 1


        # now we can execute the action and save the state
        if self.last is None and found == 1:
            self.one.start()
            self.one.update(1)
            self.one.stop()

        if self.last != found:
            if self.last is not None:
                self.actions[self.last].update(1)
                self.actions[self.last].stop()
                
            self.actions[ found ].start()

        self.actions[ found ].update( new_t )
        self.last = found

    def stop(self):
        self.two.stop()
        
    def __reversed__(self):
        return Sequence( Reverse(self.two), Reverse(self.one) )

class Spawn(IntervalAction):
    """Spawn a  new action immediately.
    You can spawn actions using:

        * the Spawn() class
        * the overriden *|* operator
        * call sprite.do() many times

    Example::

        action = Spawn( action1, Spawn( action2, action3 ) )
        sprite.do( action )

        or:

        sprite.do( action1 | action2 | action3 )

        or:

        sprite.do( action1 )
        sprite.do( action2 )
        sprite.do( action3 )
    """

    def init(self, one, two):
        """Init method

        :Parameters:
            `one` : `Action`
                The first action to execute in parallel
            `two` : `Action`
                The second action to execute in parallel
        """
        from cocos.actions.interval_actions import Delay

        one = copy.deepcopy(one)
        two = copy.deepcopy(two)
        if one.duration > two.duration:
            two = two + Delay( one.duration-two.duration )
        elif two.duration > one.duration:
            one = one + Delay( two.duration-one.duration )

        self.duration = one.duration

        self.actions = [one, two]
        self.cloned_actions = []

    def done(self):
        ret = True
        for i in self.actions:
            ret = ret and i.done()
        return ret

    def start(self):
        for a in self.actions:
            a.target = self.target
            a.start()

    def update(self, t):
        self.actions[0].update(t)
        self.actions[1].update(t)
        

    def __reversed__(self):
        return Reverse( self.actions[0]  ) | Reverse( self.actions[1] )

class Repeat(Action):
    """Repeats an action forever.

    Example::

        action = JumpBy( (200,0), 50,3,5)
        repeat = Repeat( action )
        sprite.do( repeat )

    Note: To repeat just a finite amount of time, just do action * times .
    """
    def init(self, action):
        """Init method.

        :Parameters:
            `action` : `Action` instance
                The action that will be repeated
        """
        self.original = action
        self.action = copy.deepcopy( action )

    def start(self):
        self.action.target = self.target
        self.action.start()

    def step(self, dt):
        self.action.step(dt)
        if self.action.done():
            self.action = copy.deepcopy(self.original)
            self.start()

    def done(self):
        return False


class _ReverseTime( IntervalAction ):
    """Executes an action in reverse order, from time=duration to time=0

    WARNING: Use this action carefully. This action is not
    sequenceable. Use it as the default ``__reversed__`` method
    of your own actions, but using it outside the ``__reversed``
    scope is not recommended.

    The default ``__reversed__`` method for all the `Grid3DAction` actions
    and `Camera3DAction` actions is ``_ReverseTime()``.
    """
    def init(self, other, *args, **kwargs):
        super(_ReverseTime, self).init(*args, **kwargs)
        self.other = other
        self.duration = self.other.duration
        
    def start(self):
        self.other.target = self.target
        super(_ReverseTime, self).start()
        self.other.start()
        
    def stop(self):
        super(_ReverseTime,self).stop()
    
    def update(self, t):
        self.other.update(1-t)
    
    def __reversed__(self):
        return self.other



#
# cocos2d
# http://cocos2d.org
#

import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import pyglet

from cocos.director import director
from cocos.scene import Scene
from cocos.layer import Layer
from cocos.actions import JumpBy, Lens3D, Reverse

# create a layer with an image
class BackgroundLayer( Layer ):
    def __init__( self ):
        # always call super()
        super( BackgroundLayer, self).__init__()

        # load the image form file
        self.image = pyglet.resource.image('flag.png')

    def draw( self ):
        # blit the image on every frame
        self.image.blit(0,0)

if __name__ == "__main__":
    # initialize the director,
    # enabling to resize the main window
    director.init( resizable=True)

    # enable opengl depth test
    # since we are using z-values
    director.set_depth_test()

    # create an Scene with 1 layer: BackgroundLayer
    scene = Scene( BackgroundLayer() )


    # create a Lens effect action
    #  radius: 150 pixels
    #  lens_effect: 0.7, a strong "lens". 0 means no effect at all. 1 means very strong
    #  center: center of the lens
    #  grid=(20,16): create a grid of 20 tiles x 16 tiles. More tiles will
    #     look better but the performance will decraese
    #  duration=10: 10 seconds
    lens = Lens3D( radius=150, lens_effect=0.7, center=(150,150), grid=(20,16), duration=50)

    # create a Jump action
    # Jump to the right 360 pixels doing:
    #   3 jumps
    #   of height 170 pixels
    #  in 4 seconds
    jump = JumpBy((360,0),170,3,4 )

    # do and get the cloned action of lens'
    action = scene.do( lens )

    # perform the Jump action using as target the lens effect.
    # the Jump action will modify the 'position' attribute, and
    # the lens action uses the 'position' attribute as the center of the lens
    #
    # The action Jump + Reverse(Jump) will be repeated 5 times
    scene.do( (jump + Reverse(jump)) * 5, target=action)

    # Run!
    director.run( scene )



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.actions import Rotate, Reverse, MoveBy, Delay
import pyglet
from cocos.sprite import Sprite

class TestLayer(cocos.layer.Layer):
    def __init__(self):
        super( TestLayer, self ).__init__()
        
        x,y = director.get_window_size()

        self.sprite = Sprite( 'grossini.png', (x/2, y/2) )
        self.add( self.sprite )
        
        self.sprite2 = Sprite( 'grossini.png', (x/2, y/4) )
        self.add( self.sprite2 )

        seq = Rotate( 360, 10 ) | MoveBy((x/2,0))
        self.sprite.do( seq )
        self.sprite2.do( Reverse( seq ) )
        

if __name__ == "__main__":
    director.init()
    test_layer = TestLayer ()
    main_scene = cocos.scene.Scene (test_layer)
    director.run (main_scene)



# -*- coding: utf-8 -*-
# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#

import pyglet
import cocos
from cocos.actions import ScaleTo, MoveTo, Accelerate
from cocos.sprite import Sprite
from cocos.director import director
from cocos.scene import Scene

class Bg(cocos.layer.Layer):
    def __init__(self):
        super(Bg, self).__init__()
        self.image = pyglet.resource.image('grossini.png')
        self.image.anchor_x = self.image.width / 2
        self.image.anchor_y = self.image.height / 2

    def on_enter(self):
        super(Bg, self).on_enter()
        sprite = Sprite(self.image)
        self.add(sprite)
        sprite.position = 320, 240
        sprite.do(ScaleTo(4,0))
        
        action = MoveTo((640, 480), 4) | ( ScaleTo(2,2) + ScaleTo(4,2) )

        sprite.do(action)
        
director.init()
director.run(Scene(Bg()))



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#


import cocos
from cocos.director import director
from cocos.sprite import ActionSprite
from cocos.actions import *
import pyglet
        
        

if __name__ == "__main__":
    director.init()
    test_layer = cocos.layer.ColorLayer(255,0,0,255)
    main_scene = cocos.scene.Scene (test_layer)
    test_layer.do( Delay(5) + CallFunc(director.pop) )
    director.run (main_scene)



class Rect(object):
    '''Define a rectangular area.

    Many convenience handles and other properties are also defined - all of
    which may be assigned to which will result in altering the position
    and sometimes dimensions of the Rect.

    The Rect area includes the bottom and left borders but not the top and
    right borders.
    '''
    def __init__(self, x, y, width, height):
        '''Create a Rect with the bottom-left corner at (x, y) and
        dimensions (width, height).
        '''
        self._x, self._y = x, y
        self._width, self._height = width, height

    # the following four properties will most likely be overridden in a
    # subclass
    def set_x(self, value): self._x = value
    x = property(lambda self: self._x, set_x)
    def set_y(self, value): self._y = value
    y = property(lambda self: self._y, set_y)
    def set_width(self, value): self._width = value
    width = property(lambda self: self._width, set_width)
    def set_height(self, value): self._height = value
    height = property(lambda self: self._height, set_height)
    def set_position(self, value): self._x, self._y = value
    position = property(lambda self: (self._x, self._y), set_position)
    def set_size(self, value): self._width, self._height = value
    size = property(lambda self: (self._width, self._height), set_size)

    def contains(self, x, y):
        '''Return boolean whether the point defined by x, y is inside the
        rect area.
        '''
        if x < self._x or x > self._x + self._width: return False
        if y < self._y or y > self._y + self._height: return False
        return True

    def intersects(self, other):
        '''Return boolean whether the "other" rect (an object with .x, .y,
        .width and .height attributes) overlaps this Rect in any way.
        '''
        if self._x + self._width < other.x: return False
        if other.x + other.width < self._x: return False
        if self._y + self._height < other.y: return False
        if other.y + other.height < self._y: return False
        return True

    # r/w, in pixels, y extent
    def get_top(self): return self.y + self.height
    def set_top(self, y): self.y = y - self.height
    top = property(get_top, set_top)

    # r/w, in pixels, y extent
    def get_bottom(self): return self.y
    def set_bottom(self, y): self.y = y
    bottom = property(get_bottom, set_bottom)

    # r/w, in pixels, x extent
    def get_left(self): return self.x
    def set_left(self, x): self.x = x
    left = property(get_left, set_left)

    # r/w, in pixels, x extent
    def get_right(self): return self.x + self.width
    def set_right(self, x): self.x = x - self.width
    right = property(get_right, set_right)

    # r/w, in pixels, (x, y)
    def get_center(self):
        return (self.x + self.width/2, self.y + self.height/2)
    def set_center(self, center):
        x, y = center
        self.position = (x - self.width/2, y - self.height/2)
    center = property(get_center, set_center)

    # r/w, in pixels, (x, y)
    def get_midtop(self):
        return (self.x + self.width/2, self.y + self.height)
    def set_midtop(self, midtop):
        x, y = midtop
        self.position = (x - self.width/2, y - self.height)
    midtop = property(get_midtop, set_midtop)

    # r/w, in pixels, (x, y)
    def get_midbottom(self):
        return (self.x + self.width/2, self.y)
    def set_midbottom(self, midbottom):
        x, y = midbottom
        self.position = (x - self.width/2, y)
    midbottom = property(get_midbottom, set_midbottom)

    # r/w, in pixels, (x, y)
    def get_midleft(self):
        return (self.x, self.y + self.height/2)
    def set_midleft(self, midleft):
        x, y = midleft
        self.position = (x, y - self.height/2)
    midleft = property(get_midleft, set_midleft)

    # r/w, in pixels, (x, y)
    def get_midright(self):
        return (self.x + self.width, self.y + self.height/2)
    def set_midright(self, midright):
        x, y = midright
        self.position = (x - self.width, y - self.height/2)
    midright = property(get_midright, set_midright)
 
    # r/w, in pixels, (x, y)
    def get_topleft(self):
        return (self.x, self.y + self.height)
    def set_topleft(self, position):
        x, y = position
        self.position = (x, y - self.height)
    topleft = property(get_topleft, set_topleft)
 
    # r/w, in pixels, (x, y)
    def get_topright(self):
        return (self.x + self.width, self.y + self.height)
    def set_topright(self, position):
        x, y = position
        self.position = (x - self.width, y - self.height)
    topright = property(get_topright, set_topright)
 
    # r/w, in pixels, (x, y)
    def get_bottomright(self):
        return (self.x + self.width, self.y)
    def set_bottomright(self, position):
        x, y = position
        self.position = (x - self.width, y)
    bottomright = property(get_bottomright, set_bottomright)
 
    # r/w, in pixels, (x, y)
    def get_bottomleft(self):
        return (self.x, self.y)
    def set_bottomleft(self, position):
        self.x, self.y = position
    bottomleft = property(get_bottomleft, set_bottomleft)




# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '../..'))

from operator import setslice

from cocos.director import director
from cocos.layer import *
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.actions import *
from cocos.sprite import *
from cocos.menu import *
from cocos.text import *

import pyglet
from pyglet import gl, font
from pyglet.window import key

from HUD import BackgroundLayer
import soundex
import hiscore

class ScoresLayer( ColorLayer):

    FONT_SIZE=30

    is_event_handler = True #: enable pyglet's events

    def __init__(self):

        w,h = director.get_window_size()
        super( ScoresLayer,self).__init__( 32,32,32,16, width=w, height=h-86)

        self.font_title = {}

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] = 'Edit Undo Line BRK'
        self.font_title['font_size'] = 72
        self.font_title['color'] = (204,164,164,255)
        self.font_title['anchor_y'] ='top'
        self.font_title['anchor_x'] ='center'

        title = Label('TETRICO', **self.font_title )

        title.position=(w/2.0,h)

        self.add(title,z=1)

        self.table = None

    def on_enter( self ):
        super(ScoresLayer,self).on_enter()

        scores = hiscore.hiscore.get()

        if self.table:
            self.remove_old()

        self.table =[]
        for idx,s in enumerate(scores):

            pos= Label( '%d:' % (idx+1), font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='left',
                        color=(255,255,255,255) )

            name = Label( s[1], font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='left',
                        color=(255,255,255,255) )

            score = Label( str(s[0]), font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='right',
                        color=(255,255,255,255) )

            lvl = Label( str(s[2]), font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='right',
                        color=(255,255,255,255) )

            self.table.append( (pos,name,score,lvl) )

        self.process_table()

    def remove_old( self ):
        for item in self.table:
            pos,name,score,lvl = item
            self.remove(pos)
            self.remove(name)
            self.remove(score)
            self.remove(lvl)
        self.table = None

    def process_table( self ):
        w,h = director.get_window_size()

        for idx,item in enumerate(self.table):
            pos,name,score,lvl = item

            posy = h - 100 - ( (self.FONT_SIZE+15) * idx )

            pos.position=( 5, posy)
            name.position=( 48, posy)
            score.position=( w-150, posy )
            lvl.position=( w-10, posy)

            self.add( pos, z=2 )
            self.add( name, z=2 )
            self.add( score, z=2 )
            self.add( lvl, z=2 )

    def on_key_press( self, k, m ):
        if k in (key.ENTER, key.ESCAPE, key.SPACE):
            self.parent.switch_to( 0 )
            return True

    def on_mouse_release( self, x, y, b, m ):
        self.parent.switch_to( 0 )
        return True


class OptionsMenu( Menu ):
    def __init__(self):
        super( OptionsMenu, self).__init__('TETRICO') 
        self.select_sound = soundex.load('move.mp3')

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] = 'Edit Undo Line BRK'
        self.font_title['font_size'] = 72
        self.font_title['color'] = (204,164,164,255)

        self.font_item['font_name'] = 'Edit Undo Line BRK',
        self.font_item['color'] = (32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = 'Edit Undo Line BRK'
        self.font_item_selected['color'] = (32,16,32,255)
        self.font_item_selected['font_size'] = 46

        # you can also override the font size and the colors. see menu.py for
        # more info

        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER

        items = []

        self.volumes = ['Mute','10','20','30','40','50','60','70','80','90','100']

        items.append( MultipleMenuItem(
                        'SFX volume: ', 
                        self.on_sfx_volume,
                        self.volumes,
                        int(soundex.sound_vol * 10) )
                    )
        items.append( MultipleMenuItem(
                        'Music volume: ', 
                        self.on_music_volume,
                        self.volumes,
                        int(soundex.music_player.volume * 10) )
                    )
        items.append( ToggleMenuItem('Show FPS:', self.on_show_fps, director.show_FPS) )
        items.append( MenuItem('Fullscreen', self.on_fullscreen) )
        items.append( MenuItem('Back', self.on_quit) )
        self.create_menu( items, shake(), shake_back() )

    def on_fullscreen( self ):
        director.window.set_fullscreen( not director.window.fullscreen )

    def on_quit( self ):
        self.parent.switch_to( 0 )

    def on_show_fps( self, value ):
        director.show_FPS = value

    def on_sfx_volume( self, idx ):
        vol = idx / 10.0
        soundex.sound_volume( vol )

    def on_music_volume( self, idx ):
        vol = idx / 10.0
        soundex.music_volume( vol )

class MainMenu( Menu ):

    def __init__(self):
        super( MainMenu, self).__init__('TETRICO') 

        self.select_sound = soundex.load('move.mp3')

        # you can override the font that will be used for the title and the items
        # you can also override the font size and the colors. see menu.py for
        # more info
        self.font_title['font_name'] = 'Edit Undo Line BRK'
        self.font_title['font_size'] = 72
        self.font_title['color'] = (204,164,164,255)

        self.font_item['font_name'] = 'Edit Undo Line BRK',
        self.font_item['color'] = (32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = 'Edit Undo Line BRK'
        self.font_item_selected['color'] = (32,16,32,255)
        self.font_item_selected['font_size'] = 46


        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER

        items = []

        items.append( MenuItem('New Game', self.on_new_game) )
        items.append( MenuItem('Options', self.on_options) )
        items.append( MenuItem('Scores', self.on_scores) )
        items.append( MenuItem('Quit', self.on_quit) )

        self.create_menu( items, shake(), shake_back() )

    def on_new_game(self):
        import gameview
        director.push( FlipAngular3DTransition(
            gameview.get_newgame(), 1.5 ) )

    def on_options( self ):
        self.parent.switch_to(1)


    def on_scores( self ):
        self.parent.switch_to(2)

    def on_quit(self):
        pyglet.app.exit()

if __name__ == "__main__":

    pyglet.resource.path.append('data')
    pyglet.resource.reindex()
    font.add_directory('data')

    director.init( resizable=True, width=600, height=720 )
    scene = Scene()
    scene.add( MultiplexLayer(
                    MainMenu(), 
                    OptionsMenu(),
                    ScoresLayer(),
                    ),
                z=1 ) 
    scene.add( BackgroundLayer(), z=0 )
    director.run( scene )



#
# cocos2d
# http://cocos2d.org
#
# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#

import cocos
from cocos.actions import *

# A color layer  is a Layer with the a color attribute
class HelloWorld(cocos.layer.ColorLayer):
    def __init__(self):
        # blueish color
        super( HelloWorld, self ).__init__( 64,64,224,255)

        # a cocos.text.Label is a wrapper of pyglet.text.Label
        # with the benefit of being a CocosNode
        label = cocos.text.Label('Hello, World!',
            font_name='Times New Roman',
            font_size=32,
            anchor_x='center', anchor_y='center')

        # set the label in the center of the screen
        label.position = 320,240
        self.add( label )
        
        # similar to cocos.text.Label, a cocos.sprite.Sprite
        # is a subclass of pyglet.sprite.Sprite with the befits of
        # being a CocosNode.
        sprite = cocos.sprite.Sprite('grossini.png')
        
        # sprite in the center of the screen (default is 0,0)
        sprite.position = 320,240
        
        # sprite scale attribute starts with 3 (default 1 )
        sprite.scale = 3
        
        # add the sprite as a child, but with z=1 (default is z=0).
        # this means that the sprite will be drawn on top of the label
        self.add( sprite, z=1 )

        # create a ScaleBy action that lasts 2 seconds
        scale = ScaleBy(3, duration=2)
        
        # tell the label to scale and scale back and repeat these 2 actions forever
        label.do( Repeat( scale + Reverse( scale) ) )
        
        # tell the sprite to scaleback and then scale, and repeat these 2 actions forever
        sprite.do( Repeat( Reverse(scale) + scale ) )

if __name__ == "__main__":
    # director init takes the same arguments as pyglet.window
    cocos.director.director.init()

    # We create a new layer, an instance of HelloWorld
    hello_layer = HelloWorld ()
    
    # tell the layer to perform a Rotate action in 10 seconds.
    hello_layer.do( RotateBy(360, duration=10) )

    # A scene that contains the layer hello_layer
    main_scene = cocos.scene.Scene (hello_layer)

    # And now, start the application, starting with main_scene
    cocos.director.director.run (main_scene)

    # or you could have written, without so many comments:
    #      director.run( cocos.scene.Scene( HelloWorld() ) )



#
# cocos2d:
# http://cocos2d.org
#

import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import pyglet
from pyglet.gl import *
import cocos
from cocos.director import director


class BackgroundLayer( cocos.layer.Layer ):
    def __init__(self):
        super( BackgroundLayer, self ).__init__()
        self.img = pyglet.resource.image('background_image.png')

    def on_enter(self):
        super(BackgroundLayer,self).on_enter()
        director.push_handlers(self.on_resize)

    def draw( self ):
        self.img.blit(0,0)

    def on_resize( self, width, height ):
        # change to custom projection projection
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(90, 1.0*width/height, 0.1, 3000.0)
        glMatrixMode(GL_MODELVIEW)

if __name__ == '__main__':
    director.init()
    s = cocos.scene.Scene()
    s.add( BackgroundLayer() )
    director.run( s )



# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '../..'))
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))


import cocos
from cocos.director import director

import pyglet
from pyglet import font, image
from pyglet.gl import *
from pyglet.window import key

from cocos.actions import *
from cocos.layer import *
from cocos.scenes.transitions import *
from cocos.sprite import *
from cocos import text

import demo_grid_effects


def get_color_layer( idx ):
    if idx % 2 == 0:
        return cocos.layer.ColorLayer(32,32,32,255)
    else:
        return cocos.layer.ColorLayer(32,64,200,255)

       

class TitleSubTitleLayer(cocos.layer.Layer):
    def __init__(self, title, subtitle):
        super(TitleSubTitleLayer, self).__init__()

        x, y = director.get_window_size()

        self.title = text.Label(
                title, (x/2, y/2+50), font_name = 'Gill Sans',
                font_size = 64, anchor_x='center', anchor_y='center' )
        self.add( self.title )
        
        self.subtitle = text.Label(
                subtitle, (x/2, y/2-30), font_name = 'Gill Sans',
                font_size = 44, anchor_x='center', anchor_y='center' )
        self.add( self.subtitle )
        
class BulletListLayer(cocos.layer.Layer):
    def __init__(self, title, lines):
        super(BulletListLayer, self).__init__()
        x, y = director.get_window_size()


        self.title = text.Label(
                title, (x/2, y-50), font_name = 'Gill Sans',
                font_size = 64, anchor_x='center', anchor_y='center' )
        self.add( self.title )

        start_y = (y/12)*8
        font_size = 52 / (len(lines)/2.2+1)
        font_size = min(font_size, 52)
        line_font = font.load('Gill Sans', font_size)
        tot_height = 0
        max_width = 0
        rendered_lines = []
        step = 300/ max(len(lines),1)
        i = 0
        for line in lines:
            line_text = text.Label(
                line, (x/2, y-150-step*i), font_name = 'Gill Sans',
                font_size = font_size, anchor_x='center', anchor_y='center' )
            i += 1   
            self.add( line_text )    
class TransitionControl(cocos.layer.Layer):
    is_event_handler = True
    
    def __init__(self, scenes, transitions=None):
        super(TransitionControl, self).__init__()
        
        self.transitions = transitions
        self.scenes = scenes
        for scene in scenes:
            if not self in scene.get_children():
                scene.add(self)
                
        self.scene_p = 0
        
            
    def next_scene(self):
        self.duration = None
        self.scene_p +=1 
        if self.scene_p >= len(self.scenes):
            self.scene_p = len(self.scenes)-1
        else:
            self.transition(self.transitions[self.scene_p%len(self.transitions)-1])
    
    def prev_scene(self):
        self.duration=0.5
        self.scene_p -=1 
        if self.scene_p < 0:
            self.scene_p = 0
        else:
#            self.transition()
            self.transition(self.transitions[self.scene_p%len(self.transitions)])
           
    def transition(self, transition=None):
        if transition:
            director.replace( transition(
                        self.scenes[ self.scene_p ],
                        duration = self.duration
                         )
                )
        else:
            director.replace( self.scenes[ self.scene_p ] )
        
    def on_key_press(self, keyp, mod):
        if keyp in (key.PAGEDOWN,):
            self.next_scene()
        elif keyp in (key.PAGEUP,):
            self.prev_scene()
            
class RunScene(cocos.layer.Layer):
    is_event_handler = True
    
    def __init__(self, target):
        super(RunScene, self).__init__()
        
        self.target = target
        
    def on_key_press(self, keyp, mod):
        if keyp in (key.F1,):
            director.push( self.target )    
        

if __name__ == "__main__":
    aspect = 1280 / float(800)
    director.init( resizable=True, width=640, height=480 ) 
    director.window.set_fullscreen(False)
    x,y = director.get_window_size()

    pyglet.font.add_directory('..')

    scenes = [
        cocos.scene.Scene (
            TitleSubTitleLayer("cocos2d", "a 2d game library"),
        ),
        cocos.scene.Scene (
            BulletListLayer("cocos2d", [
                "a framework for",
                "building 2D games, demos",
                "and other graphical interactive applications", 
                "developed 100% in python",
                ])
            ),
        cocos.scene.Scene (
            BulletListLayer("Games-Python", [
                "Games in an interpreted language?",
                "Depends on the game",
                "30ms is a lot",
                ])
            ),
        
        
        cocos.scene.Scene (
            BulletListLayer("Inspiration", [
                "2000 : pygame",
                "2005 : pyweek",
                "2006 : pyglet",
                "2008 : cocos2d"
                ])
            ),
        cocos.scene.Scene (
            BulletListLayer("Features", [
                "Flow Control",
                "Sprites",
                "Actions",
                "Effects",
                "Transitions",
                "Menus", 
                ])
            ),
        cocos.scene.Scene (
            BulletListLayer("Features (2)", [
                "Text / HTML",
                "Tiles"
                "Well documented",
                "Embedded python interpreter",
                "BSD License",
                "pyglet-based",
                "OpenGL",
                ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("Concepts", []).add(
                Sprite("scene_en.png", (x/2, 100))),
            ),
        cocos.scene.Scene (
            BulletListLayer("Documentation", [
                "Video Tutorial",
                "Programming guide",
                "(almost) complete API reference",
                "5KLOC of tests",
                "tests are readable samples",
                "FAQ [in progress]",

            ]),
            ),
            
        cocos.scene.Scene (
            BulletListLayer("Community", [
                "5 finished games",
                "Several projects in development",
                "Contributors from around the world",
                "Working mailing list",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("Flow Control", [
                "director.run()",
                "director.push() / pop()",
                "director.replace",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("Sprites", [
                "position",
                "scale",
                "rotation",
                "visible",
                "opacity",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("Basic Actions", [
                "MoveBy/To",
                "ScaleBy/To",
                "RotateBy/To",
                "Hide/Show",
                "FadeIn/FadeOut",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("More Actions", [
                "Reverse",
                "Sequence",
                "Repeat",
                "Spawn",
                "Accelerate, AccelDeccel,Speed",
                "Delay",
                "CallFunc",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("Effects: Theory", [
                "FrameBuffer Object",
                "PixelBuffer Object",
                "ColorBuffer",
                "Grid / Tiled",
                "Camera",
                #"PONER SCREENSHOTS / SLIDES PARA EXPLICAR TEORIA",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("More Effects", [
                "Ripple3D",
                "Lens",
                "Shaky",
                "Flip",
                "OrbitCamera",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("More Effects II", [
                "Actions",
                "Lens + Jump",
                "Reuse Grid",
            ]),
            ),
        cocos.scene.Scene (
            BulletListLayer("Coming soon", [
                "Particle system",
                "Pymunk integration",
                "Drawing primitives",
            ]),
            ),

         cocos.scene.Scene (
            TitleSubTitleLayer("cocos2d", "http://www.cocos2d.org"),
        ),
        ]

    i = 0;
    for s in scenes:
        s.add( get_color_layer(i), z=-1)
        i = i+1

    transitions = [None]*(len(scenes)-1)
    all_t = [
            'ZoomTransition',

            'FlipX3DTransition',
            'FlipY3DTransition','FlipAngular3DTransition',

            'TurnOffTilesTransition',

            'ShrinkGrowTransition',

            'FadeTRTransition', 'FadeBLTransition',
            'FadeUpTransition', 'FadeDownTransition',

            'SplitRowsTransition', 'SplitColsTransition',

            'RotoZoomTransition',

            'FadeTransition',

            'CornerMoveTransition',
            'EnvelopeTransition',

            'MoveInLTransition','MoveInRTransition',
            'MoveInBTransition','MoveInTTransition',

            'ShuffleTransition',
            'JumpZoomTransition',

            ]

    transitions = [ getattr(cocos.scenes.transitions, all_t[i % len(all_t)]) 
                for i in range(len(scenes)-1) ]
                
    TransitionControl( scenes, transitions )

    def color_name_scene(name, color):    
        return cocos.scene.Scene(
            cocos.layer.ColorLayer(*color).add(
                cocos.text.Label(
                    name, (x/2,y/2), 
                    font_name = 'Gill Sans', font_size = 64, 
                    anchor_x='center', anchor_y='center'
                )
            )
        )
    director.interpreter_locals["one"] = color_name_scene("one", (255,0,0,255))
    director.interpreter_locals["two"] = color_name_scene("two", (0,255,0,255))
    director.interpreter_locals["three"] = color_name_scene("three", (0,0,255,255))

    director.interpreter_locals["grid_scene"] = demo_grid_effects.start()
    
    
    director.run (scenes[0])




#
# HiScore
#

__all__ = ['hiscore']

class HiScoreData( object ):

    POINTS, NAME, LVL = range(3)

    HISCORE_FILENAME = 'hi_scores.txt'

    MAX = 10

    def __init__( self ):
        super(HiScoreData, self).__init__()

        self.load()

    def load(self):
        self.hi_scores = []
        try:
            f = open(self.HISCORE_FILENAME)
            for line in f.readlines():
                # ignore comments
                if line.startswith("#"):
                    continue
                (score,name,lvl) = line.split(',')
                self.hi_scores.append( (int(score),name,int(lvl) ) )
            f.close()
        except IOError:
            # file not found, no problem
            pass

    def save(self):
        try:
            f = open(self.HISCORE_FILENAME,'w')
            for i in self.hi_scores:
                f.write('%d,%s,%d\n' % ( i[0],i[1],i[2] ) )
            f.close()
        except Exception, e:
            print 'Could not save hi scores'
            print e

    def add( self, score, name, lvl):
        # safe replacement
        for l in name:
            if not l.isalnum():
                name = name.replace(l,'_')

        self.hi_scores.append( (int(score),name,int(lvl) ) )
        self.hi_scores.sort()
        self.hi_scores.reverse()

        self.hi_scores = self.hi_scores[:self.MAX]

        self.save()

    def is_in( self, score ):
        if len( self.hi_scores) < self.MAX:
            return True
        if score > self.hi_scores[-1][0]:
            return True
        return False

    def get( self, max=10 ):
        # only return the max first records
        return self.hi_scores[:max]

hiscore = HiScoreData()



class Path:
    def at(self, t):
        pass
        
class Bezier(Path):
    def __init__(self, a, b, ac, bc):
        self.a = a
        self.b = b
        self.ac = ac
        self.bc = bc
        
    def at(self, t):
        def calc(i):
            a = self.a[i]
            b = self.ac[i]
            c = self.bc[i]
            d = self.b[i]
            return ( 
                ((1-t)**3) * a + 
                3*t*((1-t)**2)*b + 
                3*(t**2)*(1-t)*c +
                (t**3)*d
                )
        return calc(0), calc(1)
        
    def __repr__(self):
        return "Bezier( (%i,%i), (%i,%i), (%i, %i), (%i,%i) )"%(
                self.a[0], self.a[0],
                self.b[0], self.b[1],
                self.ac[0], self.ac[1],
                self.bc[0], self.bc[1],

            )
    



#
# Cocos
# http://code.google.com/p/los-cocos/
#
# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#

import cocos

class HelloWorld(cocos.layer.Layer):
    is_event_handler = True
    def __init__(self):
        super( HelloWorld, self ).__init__()

        # a cocos.text.Label is a wrapper of pyglet.text.Label
        # with the benefit of being a cocosnode
        self.label = cocos.text.Label('Hi',
            font_name='Times New Roman',
            font_size=32,
            x=320, y=240,
            anchor_x='center', anchor_y='center')

        self.add( self.label )

    def on_mouse_motion( self, x, y, dx, dy ):
        self.label.element.text = '%d,%d' % (x,y)
        self.label.element.x = x
        self.label.element.y = y


if __name__ == "__main__":
    # director init takes the same arguments as pyglet.window
    cocos.director.director.init()

    # We create a new layer, an instance of HelloWorld
    hello_layer = HelloWorld ()

    # A scene that contains the layer hello_layer
    main_scene = cocos.scene.Scene (hello_layer)

    # And now, start the application, starting with main_scene
    cocos.director.director.run (main_scene)

    # or you could have written, without so many comments:
    #      director.run( cocos.scene.Scene( HelloWorld() ) )



import pyglet
from pyglet.gl import *
# This code is so you can run the samples without installing the package
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
#
import math

def sign(x): return 1 if x >= 0 else -1

class MeshSprite:
    def __init__(self, image, x_quads, y_quads):
        x_step = image.width / (x_quads)
        y_step = image.height / (y_quads)
        self.image = image
        self.texture = image.get_texture()

        w = float(image.width)/self.texture.tex_coords[3]
        h = float(image.height)/self.texture.tex_coords[7]
        vertex_points = []
        texture_points = []
        for x in range(0, x_quads):
            for y in range(0, y_quads):
                x1 = x*x_step 
                x2 = x1 + x_step
                y1 = y*y_step
                y2 = y1 + y_step
                
                vertex_points += [x1, y1, x2, y1, x2, y2, x1, y2]
                texture_points += [x1/w, y1/h, x2/w, y1/h, x2/w, y2/h, x1/w, y2/h]
                
      
        self.vertex_list = pyglet.graphics.vertex_list(x_quads*y_quads*4, "t2f", "v2i/stream")
        self.vertex_points = vertex_points[:]
        self.vertex_list.vertices = vertex_points
        self.vertex_list.tex_coords = texture_points
        self.elapsed = 0
        
    def draw(self, dt):
        self.elapsed += dt
        glEnable(self.texture.target)
        glBindTexture(self.texture.target, self.texture.id)
        glPushAttrib(GL_COLOR_BUFFER_BIT)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        for p in range(len(self.vertex_points)/2):
            x = self.vertex_points[2*p]
            y = self.vertex_points[2*p+1]
            if 0:
                scale = abs(math.cos ( float(x)/(y+0.5) + self.elapsed ) ) + 1
                x = (x-25) * scale + 25
                y = (y-50) * scale + 50
            else:
                center = (2+self.elapsed)**4
                disttoc = math.sqrt( (x-25)**2 + (y-50)**2 ) - center
                scale = min(1/abs(disttoc), 1)*10
                
                x = x + sign(x-25) * scale 
                y = y + sign(y-50) * scale 
            self.vertex_list.vertices[2*p] = 200+int(x)
            self.vertex_list.vertices[2*p+1] = 200+int(y)

        self.vertex_list.draw(pyglet.gl.GL_QUADS)
        glPopAttrib()
        glDisable(self.texture.target)
        
        
window = pyglet.window.Window()

grossini = pyglet.resource.image("grossini.png")
grossini.anchor_x = grossini.width / 2
grossini.anchor_y = grossini.height / 2
ms = MeshSprite( grossini, 15,31 )
def update(dt):
    window.clear()
    ms.draw(dt)
    
pyglet.clock.schedule_interval(update, 1/60.)

@window.event
def on_key_press(key, mods):
    ms.elapsed = 0
    
pyglet.app.run()



#
# Cocos
# http://code.google.com/p/los-cocos/
#

import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from pyglet import image
from pyglet.gl import *
from pyglet import font

from cocos.director import *
from cocos.menu import *
from cocos.scene import *
from cocos.layer import *

from operator import setslice

def printf(*args):
    sys.stdout.write(''.join([str(x) for x in args])+'\n')

class MainMenu(Menu):
    def __init__( self ):
        super( MainMenu, self ).__init__("Test Menu Items")

        # then add the items
        item1= ToggleMenuItem('ToggleMenuItem: ', self.on_toggle_callback, True )
                        
        resolutions = ['320x200','640x480','800x600', '1024x768', '320x200']
        item2= MultipleMenuItem('MultipleMenuItem: ',
                        self.on_multiple_callback,
                        resolutions,
                        0 )

        item3 = MenuItem('MenuItem', self.on_callback )

        self.create_menu( [item1,item2,item3] )


    def on_quit( self ):
        pyglet.app.exit()

    def on_multiple_callback(self, idx ):
        print 'multiple item callback', idx

    def on_toggle_callback(self, b ):
        print 'toggle item callback', b

    def on_callback(self ):
        print 'item callback'


if __name__ == "__main__":

    pyglet.font.add_directory('.')

    director.init( resizable=True)
    director.run( Scene( MainMenu() ) )
